#include "graph.h"

int createGraph(char* fileName, Tgraph* graph){
  // read data in fileName and create the corresponding graph

  FILE* fd;
  int i, j;
  
  
  graph->maxDegree = 0;
  graph->nbEdges = 0;

  if ((fd=fopen(fileName, "r"))==NULL){
    printf("ERROR: Cannot open ascii input file %s", fileName); 
    return 1;	
  }

  fscanf(fd,"%d",&(graph->nbVertices));
  printf("%d : ", graph->nbVertices);
  
  graph->nbSucc = (unsigned short*)malloc(graph->nbVertices*sizeof(unsigned short));
  graph->succ = (unsigned short**)malloc(graph->nbVertices*sizeof(unsigned short*));
  graph->isEdge = (char**)malloc(graph->nbVertices*sizeof(char*));

 
  for (i=0; i<graph->nbVertices; i++){
   // reading degree and successors of vertex i
    fscanf(fd,"%hu",&(graph->nbSucc[i]));
    graph->nbEdges+=graph->nbSucc[i];
    if(graph->maxDegree < graph->nbSucc[i]) graph->maxDegree = graph->nbSucc[i];
 
    if (feof(fd)){
      printf("ERROR: unexpected end of file\n");
      return 2;
    }
    graph->succ[i] = (unsigned short*)malloc(graph->nbSucc[i]*sizeof(unsigned short));
    graph->isEdge[i] = (char*)malloc(graph->nbVertices*sizeof(char));
    for (j=0; j<graph->nbVertices; j++) graph->isEdge[i][j]=0;
    
    for (j=0; j<graph->nbSucc[i]; j++){
      fscanf(fd,"%hu",&(graph->succ[i][j]));
      graph->isEdge[i][graph->succ[i][j]]=1;
      if (feof(fd)){
	printf("ERROR: unexpected end of file\n");
	return 2;
      }
    }
  }
  
  printf("%d\n", graph->nbEdges);
  

  
  fclose(fd);
  return 0;
}




/* WARNING: for simplicity this code does not check for End-Of-File! */
unsigned short read_word(FILE *fd)
// read a word in a vflib graph format file 
  { unsigned char b1, b2;
    
    b1=getc(fd); /* Least-significant Byte */
    b2=getc(fd); /* Most-significant Byte */

    return b1 | (b2 << 8);
  }



int createGraph_vflib(char* fileName, Tgraph* graph, int vflib_perc_lab){
  // read data in vflib graph format file fileName with vflib_perc_lab percentage of labels and create the corresponding graph

  FILE* fd;
  int i, j, u, success;
  int m, k2=0, k1=0, p=1;
  
  
  graph->nbEdges = 0;
  graph->maxDegree = 0;

  if ((fd=fopen(fileName, "rb"))==NULL)
    {
    	printf("ERROR: Cannot open ascii input file %s", fileName); 
    	return 1;	
    }

  graph->nbVertices=read_word(fd);
  printf("%d : ", graph->nbVertices);
  
  m = graph->nbVertices*vflib_perc_lab/100;
  while(p<m && k1<16)
    {
    	p*=2;
    	k1=k2;
    	k2++;
    }
   
  graph->nbSucc = malloc(graph->nbVertices*sizeof(unsigned short));
  graph->succ = malloc(graph->nbVertices*sizeof(unsigned short*));
  graph->edge_attr = malloc(graph->nbVertices*sizeof(unsigned short*));
  graph->isEdge = (char**)malloc(graph->nbVertices*sizeof(char*));
  
  
  graph->node_attr = malloc(graph->nbVertices*sizeof(unsigned short));
 
  
  /* Read the attributes of nodes */
  for(i=0; i<graph->nbVertices; i++) graph->node_attr[i]=read_word(fd)>>(16-k1);

  
  for (i=0; i<graph->nbVertices; i++)
    {
   	
    	graph->nbSucc[i]=read_word(fd);   

    	graph->nbEdges+=graph->nbSucc[i];
    	if(graph->maxDegree < graph->nbSucc[i]) graph->maxDegree = graph->nbSucc[i];
    	
    	graph->succ[i] = malloc(graph->nbSucc[i]*sizeof(unsigned short));
    	graph->edge_attr[i] = malloc(graph->nbVertices*sizeof(unsigned short));
    	graph->isEdge[i] = (char*)malloc(graph->nbVertices*sizeof(char));
    	for(u=0;u<graph->nbVertices;u++)
    	  { 
    		graph->edge_attr[i][u]=1;
    		graph->isEdge[i][u] = 0;
    	  }
    	
    	if (feof(fd))
    	  {
      		printf("ERROR: unexpected end of file\n");
      		fclose(fd);
      		return 2;
    	  }
    
    	for (j=0; j<graph->nbSucc[i]; j++)
    	  {
      		success=read_word(fd);
      		if (feof(fd))
      		  {
			printf("ERROR: unexpected end of file\n");
			fclose(fd);
			return 2;
      		  }
      		
      		graph->succ[i][j]=success;
      		graph->isEdge[i][success] = 1;
      		
      		/* Read the edge attribute and insert in the matrix that holds the attributes of edges */
      		graph->edge_attr[i][success]=read_word(fd)>>(16-k1);
    	  }
    }
  
  printf("%d\n", graph->nbEdges);
  

  fclose(fd);
  return 0;
}


int createGraph_vflib_unlab(char* fileName, Tgraph* graph){
  // read data in vflib graph format file fileName without labels and create the corresponding graph

  FILE* fd;
  int i, j, u, success;
  
  
  graph->nbEdges = 0;
  graph->maxDegree = 0;

  if ((fd=fopen(fileName, "rb"))==NULL)
    {
    	printf("ERROR: Cannot open ascii input file %s", fileName); 
    	fclose(fd);
    	return 1;	
    }

  graph->nbVertices=read_word(fd);
  printf("%d : ", graph->nbVertices);

  graph->nbSucc = malloc(graph->nbVertices*sizeof(unsigned short));
  graph->succ = malloc(graph->nbVertices*sizeof(unsigned short*));
  graph->edge_attr = malloc(graph->nbVertices*sizeof(unsigned short*));
  graph->isEdge = (char**)malloc(graph->nbVertices*sizeof(char*));
  
  
  graph->node_attr = malloc(graph->nbVertices*sizeof(unsigned short));
 
  
  /* Read the attributes of nodes */
  for(i=0; i<graph->nbVertices; i++) 
    {
      read_word(fd);
      graph->node_attr[i]=1;
    }

  
  for (i=0; i<graph->nbVertices; i++)
    {
   	
    	graph->nbSucc[i]=read_word(fd);   

    	graph->nbEdges+=graph->nbSucc[i];
    	if(graph->maxDegree < graph->nbSucc[i]) graph->maxDegree = graph->nbSucc[i];
    	
    	graph->succ[i] = malloc(graph->nbSucc[i]*sizeof(unsigned short));
    	graph->edge_attr[i] = malloc(graph->nbVertices*sizeof(unsigned short));
    	graph->isEdge[i] = (char*)malloc(graph->nbVertices*sizeof(char));
    	for(u=0;u<graph->nbVertices;u++)
    	  { 
    		graph->edge_attr[i][u]=1;
    		graph->isEdge[i][u] = 0;
    	  }
    	
    	if (feof(fd))
    	  {
      		printf("ERROR: unexpected end of file\n");
      		fclose(fd);
      		return 2;
    	  }
    
    	for (j=0; j<graph->nbSucc[i]; j++)
    	  {
      		success=read_word(fd);
      		if (feof(fd))
      		  {
			printf("ERROR: unexpected end of file\n");
			fclose(fd);
			return 2;
      		  }
      		
      		graph->succ[i][j]=success;
      		graph->isEdge[i][success] = 1;
      		
      		/* Read the edge attribute and insert in the matrix that holds the attributes of edges */
      		read_word(fd);
    	  }
    }
  
  printf("%d\n", graph->nbEdges);
  

  fclose(fd);
  return 0;
}


void deleteGraph_vflib(Tgraph* graph){
// delete a vflib graph

  int i;
  
  for (i=0; i<graph->nbVertices; i++)
    {
    	free(graph->succ[i]);
    	free(graph->edge_attr[i]);
    	free(graph->isEdge[i]);
    }
    
  free(graph->nbSucc);
  free(graph->succ);
  free(graph->edge_attr);
  free(graph->isEdge);
  free(graph->node_attr);
}


void deleteGraphcopy(Tgraph* graph){
// delete a partial copy of a graph

  int i;
  
  for (i=0; i<graph->nbVertices; i++)
    {
    	free(graph->succ[i]);
    }
    
  free(graph->nbSucc);
  free(graph->succ);
}



int createGraph_undir(char* fileName, Tgraph* graph){
  // read data in fileName and create the corresponding undirected graph

  FILE* fd;
  int i, j, u, k;
  unsigned short success, nb;
  
  
  graph->maxDegree = 0;
  graph->nbEdges = 0;

  if ((fd=fopen(fileName, "r"))==NULL){
    printf("ERROR: Cannot open ascii input file %s", fileName); 
    return 1;	
  }

  fscanf(fd,"%d",&(graph->nbVertices));
  printf("%d : ", graph->nbVertices);
  
  graph->nbSucc = (unsigned short*)malloc(graph->nbVertices*sizeof(unsigned short));
  graph->succ = (unsigned short**)malloc(graph->nbVertices*sizeof(unsigned short*));
  graph->isEdge = (char**)malloc(graph->nbVertices*sizeof(char*));

  for(i=0; i<graph->nbVertices; i++) 
    {
      graph->nbSucc[i]=0;

      graph->isEdge[i] = (char*)malloc(graph->nbVertices*sizeof(char));
      
      for(u=0;u<graph->nbVertices;u++) graph->isEdge[i][u] = 0;
    }
 
  for (i=0; i<graph->nbVertices; i++){
   // reading degree and successors of vertex i
    fscanf(fd,"%hu",&nb);
 
    if (feof(fd)){
      printf("ERROR: unexpected end of file\n");
      return 2;
    }
    
    for (j=0; j<nb; j++)
    	  {
      		fscanf(fd,"%hu",&success);
      		if (feof(fd))
      		  {
			printf("ERROR: unexpected end of file\n");
			fclose(fd);
			return 2;

      		  }
      		  
      		if(i!=success && graph->isEdge[i][success] == 0)	
        	  {
        		graph->nbEdges++;
        		graph->isEdge[i][success] = 1;
        		graph->isEdge[success][i] = 1;
        		graph->nbSucc[i]++;
        		graph->nbSucc[success]++;

        	  }
        	if (feof(fd)){
			printf("ERROR: unexpected end of file\n");
			return 2;
      		}
    	  }

    }
  
  for (i=0; i<graph->nbVertices; i++)
    {
    	graph->succ[i] = (unsigned short*)calloc(graph->nbSucc[i],sizeof(unsigned short));
    	if (graph->nbSucc[i]>graph->maxDegree) graph->maxDegree=graph->nbSucc[i];
    	k=0;

    	for (j=0; j<graph->nbVertices; j++)
    	  {
  		if(graph->isEdge[i][j] == 1) 
  		  {
  		  	graph->succ[i][k]=j;
  		  	k++;
  		  }
  	  }
    }
  
  
  printf("%d\n", graph->nbEdges);
  

  
  fclose(fd);
  return 0;
}



int createGraph_vflib_undir(char* fileName, Tgraph* graph, int vflib_perc_lab){
  // read data in a vflib graph format file fileName with vflib_perc_lab percentage of labels and create the corresponding undirected graph

  FILE* fd;
  int i, j, u, nb, k;
  unsigned short success;
  int m, k2=0, k1=0, p=1;
  
  
  graph->nbEdges = 0;
  graph->maxDegree = 0;

  if ((fd=fopen(fileName, "rb"))==NULL)
    {
    	printf("ERROR: Cannot open ascii input file %s", fileName); 
    	fclose(fd);
    	return 1;	
    }

  graph->nbVertices=read_word(fd);
  printf("%d : ", graph->nbVertices);
  
  m = graph->nbVertices*vflib_perc_lab/100;
  while(p<m && k1<16)
    {
    	p*=2;
    	k1=k2;
    	k2++;
    }
   
  graph->nbSucc = malloc(graph->nbVertices*sizeof(unsigned short));
  graph->succ = malloc(graph->nbVertices*sizeof(unsigned short*));
  graph->edge_attr = malloc(graph->nbVertices*sizeof(unsigned short*));
  graph->isEdge = (char**)malloc(graph->nbVertices*sizeof(char*));
  
  
  graph->node_attr = malloc(graph->nbVertices*sizeof(unsigned short));
  
  for(i=0; i<graph->nbVertices; i++) 
    {
      graph->nbSucc[i]=0;
      
      graph->edge_attr[i] = malloc(graph->nbVertices*sizeof(unsigned short));
      
      graph->isEdge[i] = (char*)malloc(graph->nbVertices*sizeof(char));
      
      for(u=0;u<graph->nbVertices;u++)
    	  { 
    		graph->edge_attr[i][u]=1;
    		graph->isEdge[i][u] = 0;
    	  }
    }
 
  
  /* Read the attributes of nodes */
  for(i=0; i<graph->nbVertices; i++) graph->node_attr[i]=read_word(fd)>>(16-k1);

  
  for (i=0; i<graph->nbVertices; i++)
    {
   	
    	nb=read_word(fd); 

    	if (feof(fd))
    	  {
      		printf("ERROR: unexpected end of file\n");
      		fclose(fd);
      		return 2;
    	  }
    
    	for (j=0; j<nb; j++)
    	  {
      		success=read_word(fd);
      		if (feof(fd))
      		  {
			printf("ERROR: unexpected end of file\n");
			fclose(fd);
			return 2;
      		  }
      		  
      		if(i!=success && graph->isEdge[i][success] == 0)	
        	  {
        		graph->nbEdges++;
        		graph->isEdge[i][success] = 1;
        		graph->isEdge[success][i] = 1;
        		graph->nbSucc[i]++;
        		graph->nbSucc[success]++;
        	  }
      		
      		
      		/* Read the edge attribute and insert in the matrix that holds the attributes of edges */
      		graph->edge_attr[i][success]=read_word(fd)>>(16-k1);
      		graph->edge_attr[success][i]=graph->edge_attr[i][success];
    	  }
    }
  
  
  for (i=0; i<graph->nbVertices; i++)
    {
    	graph->succ[i] = (unsigned short*)calloc(graph->nbSucc[i],sizeof(unsigned short));
    	if (graph->nbSucc[i]>graph->maxDegree) graph->maxDegree=graph->nbSucc[i];
    	k=0;
    	for (j=0; j<graph->nbVertices; j++)
    	  {
  		if(graph->isEdge[i][j] == 1) 
  		  {
  		  	graph->succ[i][k]=j;
  		  	k++;
  		  }
  	  }
    }
  
  printf(" %d\n", graph->nbEdges);
  
  fclose(fd);
  return 0;
}



int createGraph_vflib_undir_unlab(char* fileName, Tgraph* graph){
  // read data in vflib graph format file fileName without labels and create the corresponding undirected graph

  FILE* fd;
  int i, j, u, nb, k;
  unsigned short success;
  
  
  graph->nbEdges = 0;
  graph->maxDegree = 0;

  if ((fd=fopen(fileName, "rb"))==NULL)
    {
    	printf("ERROR: Cannot open ascii input file %s", fileName); 
    	fclose(fd);
    	return 1;	
    }

  graph->nbVertices=read_word(fd);
  printf("%d : ", graph->nbVertices);
   
  graph->nbSucc = malloc(graph->nbVertices*sizeof(unsigned short));
  graph->succ = malloc(graph->nbVertices*sizeof(unsigned short*));
  graph->edge_attr = malloc(graph->nbVertices*sizeof(unsigned short*));
  graph->isEdge = (char**)malloc(graph->nbVertices*sizeof(char*));
  
  
  graph->node_attr = malloc(graph->nbVertices*sizeof(unsigned short));
  
  for(i=0; i<graph->nbVertices; i++) 
    {
      graph->nbSucc[i]=0;
      
      graph->edge_attr[i] = malloc(graph->nbVertices*sizeof(unsigned short));
      
      graph->isEdge[i] = (char*)malloc(graph->nbVertices*sizeof(char));
      
      for(u=0;u<graph->nbVertices;u++)
    	  { 
    		graph->edge_attr[i][u]=1;
    		graph->isEdge[i][u] = 0;
    	  }
    }
 
  
  /* Read the attributes of nodes */
  for(i=0; i<graph->nbVertices; i++) 
    {
      read_word(fd);
      graph->node_attr[i]=1;
    }

  
  for (i=0; i<graph->nbVertices; i++)
    {
   	
    	nb=read_word(fd); 

    	if (feof(fd))
    	  {
      		printf("ERROR: unexpected end of file\n");
      		fclose(fd);
      		return 2;
    	  }
    
    	for (j=0; j<nb; j++)
    	  {
      		success=read_word(fd);
      		if (feof(fd))
      		  {
			printf("ERROR: unexpected end of file\n");
			fclose(fd);
			return 2;
      		  }
      		  
      		if(i!=success && graph->isEdge[i][success] == 0)	
        	  {
        		graph->nbEdges++;
        		graph->isEdge[i][success] = 1;
        		graph->isEdge[success][i] = 1;
        		graph->nbSucc[i]++;
        		graph->nbSucc[success]++;
        	  }
      		
      		
      		/* Read the edge attribute and insert in the matrix that holds the attributes of edges */
      		read_word(fd);
      		graph->edge_attr[i][success]= 1;
      		graph->edge_attr[success][i]=1;
    	  }
    }
  
  
  for (i=0; i<graph->nbVertices; i++)
    {
    	graph->succ[i] = (unsigned short*)calloc(graph->nbSucc[i],sizeof(unsigned short));
    	if (graph->nbSucc[i]>graph->maxDegree) graph->maxDegree=graph->nbSucc[i];
    	k=0;
    	for (j=0; j<graph->nbVertices; j++)
    	  {
  		if(graph->isEdge[i][j] == 1) 
  		  {
  		  	graph->succ[i][k]=j;
  		  	k++;
  		  }
  	  }
    }
  
  printf(" %d\n", graph->nbEdges);
  
  fclose(fd);
  return 0;
}


void remove_edge(Tgraph* graph, int x, int y)
// remove the edge {x,y} from graph
{
  int i,j;
  i=0; 
  while(i < graph->nbSucc[x] && graph->succ[x][i] != y)
  	i++;
  if(i >= graph->nbSucc[x])
  	printf("(%d,%d) is not an edge\n", x, y);
  	
  j=0; 
  while(j < graph->nbSucc[y] && graph->succ[y][j] != x)
  	j++;
  if(j >= graph->nbSucc[y])
  	printf("(%d,%d) is not an edge\n", y, x);
  	
  if(i < graph->nbSucc[x] && j < graph->nbSucc[y])
   {
     graph->nbSucc[x]--;
     graph->succ[x][i] = graph->succ[x][graph->nbSucc[x]]; 
     
     graph->nbSucc[y]--;
     graph->succ[y][j] = graph->succ[y][graph->nbSucc[y]]; 
      
    graph->isEdge[x][y] = 0;
    graph->isEdge[y][x] = 0;
   }
}

void remove_edgesofx(Tgraph* graph, int x, int* xremoved, int* nb_xremoved)
// remove all edges including x from graph, store all removes edges in xremoved and update nb_xremoved the number of edges stored in xremoved
{
  int i,j,y;
  for(i=0; i < graph->nbSucc[x]; i++)
   { 
	y = graph->succ[x][i]; 
	j=0;
  	while(j < graph->nbSucc[y] && graph->succ[y][j] != x)
  		j++;
  	if(j >= graph->nbSucc[y])
  	  {
  		printf("pb : %d is not sucessor of %d\n",x,y);
  		exit(1);
  	  }
  	else
  	{
  	  graph->nbSucc[y]--;
     	  graph->succ[y][j] = graph->succ[y][graph->nbSucc[y]]; 
     	  xremoved[nb_xremoved[0]++] = x;
     	  xremoved[nb_xremoved[0]++] = y;
     	}
   }
  graph->nbSucc[x]=0;
  
}

void restore_edgesofx(Tgraph* graph, int x, int y)
// restore the edge {x,y} in graph
{
  graph->succ[x][graph->nbSucc[x]++] = y;

  graph->succ[y][graph->nbSucc[y]++] = x;
}


void remove_arc(Tgraph* graph, int x, int y)
// remove the arc (x,y) from graph
{
  int i;
  i=0; 
  while(i < graph->nbSucc[x] && graph->succ[x][i] != y)
  	i++;
  if(i >= graph->nbSucc[x])
  	printf("(%d,%d) is not an arc\n", x, y);
	
  else
   {
     graph->nbSucc[x]--;
     graph->succ[x][i] = graph->succ[x][graph->nbSucc[x]];
     
     graph->isEdge[x][y] = 0;
   }
}



void connectedcomponentincludingx (Tgraph* graph, int x, int* inccx, int* nbInccx, int* outccx, int* nbOutccx)
/* 
compute the connected component including x in graph 
inccx is the set of vertices included in the connected component
nbInccx is the number of vertices included in the connected component
outccx is the set of vertices not included in the connected component
nbOutccx is the number of vertices not included in the connected component 
*/
{
  int i;
  int * pile, sommet;
  int v;
  char * marque;
  
  pile = (int *) malloc (sizeof(int) * graph->nbVertices);
  marque = (char *) calloc (graph->nbVertices, sizeof(char));
  
  marque[x] = 1;

  sommet = 0;
  pile[sommet++] = x;

  while (sommet != 0)
    {
      v = pile[--sommet];

      for(i = 0 ; i < graph->nbSucc[v]; i++)
      {
        if (! marque[graph->succ[v][i]])
        {
          pile[sommet++] = graph->succ[v][i];
          marque[graph->succ[v][i]] = 1;
        }
      }
    }
  
  nbInccx[0] = 0;
  nbOutccx[0] = 0;
  
  for(i=0; i<graph->nbVertices; i++)
   {
     if(marque[i])
      {
        inccx[nbInccx[0]++] = i; 
      }
     else
      {
        outccx[nbOutccx[0]++] = i; 
      }
   }
  
  free (pile);
  free(marque);
}



int isconnectedcomponent (Tgraph* graph, char* inccx)
/* return 1 if the subgraph of graph induced by the vertices in inccx is connected, 0 otherwise */
{
  int i;
  int * pile;
  char * marque;
  int v, sommet, nb_marque = 0, t_inccx = 0;
  
  pile = (int *) malloc (sizeof(int) * graph->nbVertices);
  marque = (char *) calloc (graph->nbVertices, sizeof(char));
  
  for(i = 0; i < graph->nbVertices; i++)
    {
      if(inccx[i])
      	t_inccx++;
    }
    
  v = 0;
  while(v < graph->nbVertices && ! inccx[v])
  	v++;
  	
  if(v >= graph->nbVertices)
   {
     printf("isconnectedcomponent : no node in the connected component\n");
     return 0;
   }
  
  marque[v] = 1;
  nb_marque++;
  
  sommet = 0;
  pile[sommet++] = v;

  while (sommet != 0)
    {
      v = pile[--sommet];

      for(i = 0 ; i < graph->nbSucc[v]; i++)
      {
        if (! marque[graph->succ[v][i]] && inccx[graph->succ[v][i]])
        {
          pile[sommet++] = graph->succ[v][i];
          marque[graph->succ[v][i]] = 1;
          nb_marque++;
        }
      }
    }
 
 
 
 if(nb_marque < t_inccx)
   {
  	printf("not connected\none component: ");
  	for(i = 0; i < graph->nbVertices; i++)
	    {
	      if(inccx[i] && marque[i])
	      	printf("%d ", i);
	    }
	
	printf("\nnodes not in the first component: ");
  	for(i = 0; i < graph->nbVertices; i++)
	    {
	      if(inccx[i] && ! marque[i])
	      	printf("%d ", i);
	    }
	printf("\n");
	
	free (pile);
 	free (marque);
  	return 0;
   }
 
 free (pile);
 free (marque);
 return 1;
}

int isconnectedcomponent_filtering (Tgraph* graph, char* inccx)
/* return 1 if the subgraph in graph induced by vertices in inccx is connected, 0 otherwise */
{
  int i;
  int * pile;
  char * marque;
  int v, sommet, nb_marque = 0, t_inccx = 0;
  
  pile = (int *) malloc (sizeof(int) * graph->nbVertices);
  marque = (char *) calloc (graph->nbVertices, sizeof(char));
  
  for(i = 0; i < graph->nbVertices; i++)
    {
      if(inccx[i])
      	t_inccx++;
    }
    
  v = 0;
  while(v < graph->nbVertices && ! inccx[v])
  	v++;
  
  marque[v] = 1;
  nb_marque++;
  
  sommet = 0;
  pile[sommet++] = v;

  while (sommet != 0)
    {
      v = pile[--sommet];

      for(i = 0 ; i < graph->nbSucc[v]; i++)
      {
        if (! marque[graph->succ[v][i]] && inccx[graph->succ[v][i]])
        {
          pile[sommet++] = graph->succ[v][i];
          marque[graph->succ[v][i]] = 1;
          nb_marque++;
        }
      }
    }
 
 
 
 if(nb_marque < t_inccx)
   {
	free (pile);
 	free (marque);
  	return 0;
   }
 
 free (pile);
 free (marque);
 return 1;
}


void copy_graph(Tgraph* graph, Tgraph* copy)
//create a partial copy of graph
{
  int i,j;
  
  copy->maxDegree = graph->maxDegree;
  copy->nbEdges = graph->nbEdges;
  copy->nbVertices = graph->nbVertices;
  
  copy->nbSucc = (unsigned short*)malloc(copy->nbVertices*sizeof(unsigned short));
  copy->succ = (unsigned short**)malloc(copy->nbVertices*sizeof(unsigned short*));

 
  for (i=0; i<graph->nbVertices; i++){
    copy->nbSucc[i] = graph->nbSucc[i];
    
    	
    copy->succ[i] = (unsigned short*)malloc(copy->nbSucc[i]*sizeof(unsigned short));
    
    for (j=0; j<graph->nbSucc[i]; j++){
      copy->succ[i][j] = graph->succ[i][j];
    }
  }
}



