#ifndef GRAPH_H
#define GRAPH_H

#include <string.h>
#include <stdio.h>
#include <stdlib.h>


typedef struct{
  int nbVertices; // Number of vertices
  int nbEdges; // Number of edges
  unsigned short* node_attr;
  
  int maxDegree;  // Degree max among all edges
  unsigned short* nbSucc;    // nbSucc[i] = number of successors of x_i
  unsigned short** succ;     // succ[i][j] = j-th successor of x_i (j in [0..nbSucc[i]-1])
  char** isEdge;   // isEdge[i][j] = 0 if x_i and x_j are not adjacent; 1 otherwise
  unsigned short** edge_attr;
} Tgraph;



int createGraph(char* fileName, Tgraph* graph);
  // read data in fileName and create the corresponding graph


int createGraph_vflib(char* fileName, Tgraph* graph, int vflib_perc_lab);
  // read data in vflib graph format file fileName with vflib_perc_lab percentage of labels and create the corresponding graph



int createGraph_vflib_unlab(char* fileName, Tgraph* graph);
  // read data in vflib graph format file fileName without labels and create the corresponding graph

 

void deleteGraph_vflib(Tgraph* graph);
  // delete a vflib graph



void deleteGraphcopy(Tgraph* graph);
  // delete a partial copy of a graph



int createGraph_undir(char* fileName, Tgraph* graph);
  // read data in fileName and create the corresponding undirected graph



int createGraph_vflib_undir(char* fileName, Tgraph* graph, int vflib_perc_lab);
  // read data in a vflib graph format file fileName with vflib_perc_lab percentage of labels and create the corresponding undirected graph



int createGraph_vflib_undir_unlab(char* fileName, Tgraph* graph);
  // read data in vflib graph format file fileName without labels and create the corresponding undirected graph



void remove_edge(Tgraph* graph, int x, int y);
// remove the edge {x,y} from graph



void remove_edgesofx(Tgraph* graph, int x, int* xremoved, int* nb_xremoved);
// remove all edges including x from graph, store all removes edges in xremoved and update nb_xremoved the number of edges stored in xremoved



void restore_edgesofx(Tgraph* graph, int x, int y);
// restore the edge {x,y} in graph



void remove_arc(Tgraph* graph, int x, int y);
// remove the arc (x,y) from graph



void connectedcomponentincludingx (Tgraph* graph, int x, int* inccx, int* nbInccx, int* outccx, int* nbOutccx);
/* 
compute the connected component including x in graph 
inccx is the set of vertices included in the connected component
nbInccx is the number of vertices included in the connected component
outccx is the set of vertices not included in the connected component
nbOutccx is the number of vertices not included in the connected component 
*/



int isconnectedcomponent (Tgraph* graph, char* inccx);
/* return 1 if the subgraph of graph induced by the vertices in inccx is connected, 0 otherwise */


int isconnectedcomponent_filtering (Tgraph* graph, char* inccx);
/* return 1 if the subgraph in graph induced by vertices in inccx is connected, 0 otherwise */


void copy_graph(Tgraph* graph, Tgraph* copy);
//create a partial copy of graph

#endif

