#include "heuristics.h"


int choose_node_mindom(Tdomain* D, Tgraph* Gp, int prof)
// return the vertex of Gp with the fewest values in its not empty domain at depth prof in the search tree or -1 if all domains are empty 
{
  int minDom=-1,u,min;
  
  for(u=0; u<Gp->nbVertices; u++)
    {
	if(D->current_sol[u]==-1 && D->nbVal_prof[prof][u]>0)
	  {
		if((minDom<0) || (D->nbVal_prof[prof][u] < min)) 
		 {
		  	minDom=u;
		  	min=D->nbVal_prof[prof][u];
		 }
	  }
    }
  return minDom;
}


int choose_node_mindom_cc(Tdomain* D, Tgraph* Gp, int prof, char* connected)
// return the vertex of Gp included in the set connected with the fewest values in its not empty domain at depth prof in the search tree or -1 if all domains of vertices in connected are empty
{
  int minDom=-1,u,min;
  
  for(u=0; u<Gp->nbVertices; u++)
    {
	if(D->current_sol[u]==-1 && connected[u] && D->nbVal_prof[prof][u]>0)
	  {
		if((minDom<0) || (D->nbVal_prof[prof][u] < min)) 
		 {
		  	minDom=u;
		  	min=D->nbVal_prof[prof][u];
		 }
	  }
    }
  return minDom;
}




int choose_value_gmatch_mindiff(Tdomain* D, int* Gp_deg, int* Gt_deg, int minDom, int prof)
// return the vertex in Gt matched to the vertex minDom in Gp if any, or the vertex of Gt in the domain of minDom at the depth prof in the search tree with the closest degree if any, or -2 otherwise
{
  if(D->nbVal_prof[prof][minDom]>0)
    {
    	if(D->globalMatching[minDom]>=0) return D->globalMatching[minDom];
    	else
    	  {
    		int u, v,ppdiff,diff,x;
    		u=D->firstVal[minDom];
	    	v=D->val[u];
	    	if(Gp_deg[minDom]>=Gt_deg[v]) ppdiff=Gp_deg[minDom] - Gt_deg[v];
	    	else ppdiff=Gt_deg[v] - Gp_deg[minDom];
	    	u++;
	    	while(u<D->firstVal[minDom]+D->nbVal_prof[prof][minDom])
	    	  {
	    	  	x=D->val[u];
	    	  	if(Gp_deg[minDom]>=Gt_deg[x]) diff=Gp_deg[minDom] - Gt_deg[x];
	    		else diff=Gt_deg[x] - Gp_deg[minDom];
	    		if(diff < ppdiff) 
	    		  {
	    		    v=x;
	    		    ppdiff=diff;
	    		  }
	    		u++;
	    	  }
	     	return v;
	   }
     }
  return -2;
}


