#include<string.h>

#include "hopcroft-karp.h"


void addToDelete(int u, int* list, int* nb, int* marked){
	if (marked[u]<toBeDeleted){
		list[(*nb)++]=u; 
		marked[u]=toBeDeleted; 
	}
}

int updateMatching(
				   int sizeOfU, // number of elements in U
				   int sizeOfV, // number of elements in V
				   Tdomain* D, // for each element u of U, D represents the elements of V compatible with u at a given depth in the search tree
				   int prof,  // depth in the search tree 
				   int*  matchedWithU // matchedWithU[i] = element of V matched with the ith element of U
){
   // compute a largest matching matchedWithU and return the size of the matching

	int nbMatched = 0; // number of elements of U that are matched to a different element of V
	int matchedWithV[sizeOfV];
	int nbPred[sizeOfV]; // nbPred[i] = nb of predecessors of the ith element of V in the DAG
	int pred[sizeOfV][sizeOfU]; // pred[i][j] = jth predecessor the ith element of V in the DAG
	int nbSucc[sizeOfU]; // nbSucc[i] = nb of successors of the ith element of U in the DAG
	int succ[sizeOfU][sizeOfV]; // succ[i][j] = jth successor of the ith element of U in the DAG
	int listV[sizeOfV], listU[sizeOfU], listDV[sizeOfV], listDU[sizeOfU];
	int nbV, nbU, nbDV, nbDU;
	int i,j,k,stop,u,v,w;
	int markedV[sizeOfV], markedU[sizeOfU];
	// markedX[i]=white if X[i] is not in the DAG
	// markedX[i]=grey if X[i] has been added to the DAG, but not its successors
	// markedX[i]=black if X[i] and its successors have been added to the DAG
	// markedX[i]=toBeDeleted if X[i] must be deleted from the DAG
	// markedX[i]=deleted if X[i] has been deleted from the DAG

	
	memset(matchedWithV,-1,sizeOfV*sizeof(int));
	for (u=0; u<sizeOfU; u++)
		if (matchedWithU[u] >= 0){
			nbMatched++;
			matchedWithV[matchedWithU[u]]=u;
		}
			
	for (u=0; u<sizeOfU; u++){
		if (matchedWithU[u]<0){
			for (i=D->firstVal[u]; i<D->firstVal[u]+D->nbVal_prof[prof][u]; i++)
				if (matchedWithV[D->val[i]]<0){// match u with v
					v=D->val[i];
					matchedWithU[u]=v; matchedWithV[v]=u; nbMatched++;
					break;
				}
		}
	}
	
	// Find augmenting paths in the DAG 
	while (nbMatched < sizeOfU){ 
		// step 1 : build the DAG
		memset(markedU,white,sizeOfU*sizeof(int));
		memset(nbSucc,0,sizeOfU*sizeof(int));
		memset(markedV,white,sizeOfV*sizeof(int));
		memset(nbPred,0,sizeOfV*sizeof(int));
		//first layer of the DAG from the free nodes of U
		nbV=0;
		for (u=0; u<sizeOfU; u++){
			if (matchedWithU[u]==-1){// u is a free node of U
				markedU[u]=black;
				for (i=D->firstVal[u]; i<D->firstVal[u]+D->nbVal_prof[prof][u]; i++){// add edge (u,v) to the DAG
					v=D->val[i];
					pred[v][nbPred[v]++]=u;
					succ[u][nbSucc[u]++]=v;
					if (markedV[v]==white){// first time v is added to the DAG
						markedV[v]=grey; listV[nbV++]=v;
					}
				} 
			}
		}
		stop=0;
		while ((stop==0) && (nbV>0)){
			// build next layer from nodes of V to nodes of U
			nbU=0;
			for (i=0; i<nbV; i++){
				v=listV[i];
				markedV[v]=black;
				u=matchedWithV[v]; 
				if (markedU[u]==white){// edge (v,u) belongs to the DAG
					markedU[u]=grey; listU[nbU++]=u;
				}
			}
			// build next layer from nodes of U to nodes of V
			nbV=0;
			for (j=0; j<nbU; j++){
				u=listU[j];
				markedU[u]=black;
				if(D->current_sol[u]==-1)
				{
				  for (i=D->firstVal[u];i<D->firstVal[u]+D->nbVal_prof[prof][u];i++){
					v=D->val[i]; 
					if (markedV[v]!=black){// add edge (u,v) to the DAG
						pred[v][nbPred[v]++]=u;
						succ[u][nbSucc[u]++]=v;
						if (markedV[v]==white){// first time v is added to the DAG
							markedV[v]=grey; listV[nbV++]=v;
						}
						if (matchedWithV[v]==-1) stop=1; // we have found a free node !
					}
				  }
			     	}
			       else if (D->current_sol[u]>=0)
			         {
			           	v=D->current_sol[u]; 
					if (markedV[v]!=black){// add edge (u,v) to the DAG
						pred[v][nbPred[v]++]=u;
						succ[u][nbSucc[u]++]=v;
						if (markedV[v]==white){// first time v is added to the DAG
							markedV[v]=grey; listV[nbV++]=v;
						}
						if (matchedWithV[v]==-1) stop=1; // we have found a free node !
					}
				 }
			}
		}
	
	
		if(nbV==0) return nbMatched;
		
		// step 2: look for augmenting paths
		for (k=0; k<nbV; k++){ 
			v=listV[k];
			if ((matchedWithV[v]==-1) && (nbPred[v]>0)){// v is the final node of an augmenting path
				int path[sizeOfU+sizeOfV];
				int length = 1;
				path[0]=v;
				nbDV=0; nbDU=0;
				addToDelete(v,listDV,&nbDV,markedV);
				do{
					u=pred[v][0]; // (u,v) belongs to the augmenting path
					path[length++]=u;
					addToDelete(u,listDU,&nbDU,markedU);
					if (matchedWithU[u]!=-1){// u is not the initial node of the augmenting path
						v=matchedWithU[u]; // (v,u) belongs to the augmenting path
						path[length++]=v;
						addToDelete(v,listDV,&nbDV,markedV);
					}
				} while (matchedWithU[u]!=-1);
				
				// delete nodes of listDV and listDU
				while ((nbDV>0) || (nbDU>0)){
					while (nbDV>0){ // delete v
						v=listDV[--nbDV]; markedV[v]=deleted;
						u=matchedWithV[v];
						if (u!=-1) addToDelete(u,listDU,&nbDU,markedU);
						for (i=0; i<nbPred[v]; i++){
							u=pred[v][i]; // delete edge (u,v)
							for (j=0; ((j<nbSucc[u]) && (v!=succ[u][j])); j++);
							succ[u][j]=succ[u][--nbSucc[u]];
							if (nbSucc[u]==0) addToDelete(u,listDU,&nbDU,markedU);
						}
					}
					while (nbDU>0){// delete u
						u = listDU[--nbDU]; markedU[u]=deleted;
						v=matchedWithU[u];
						if (v!=-1) addToDelete(v,listDV,&nbDV,markedV);
						j=0;
						for (i=0; i<nbSucc[u]; i++){// delete edge (u,v)
							v=succ[u][i];
							for (j=0; ((j<nbPred[v]) && (u!=pred[v][j])); j++);
							pred[v][j]=pred[v][--nbPred[v]];
							if (nbPred[v]==0) addToDelete(v,listDV,&nbDV,markedV);
						}
					}
				}
				
				// Update the matching wrt the augmenting path
				while (length>1){
					u=path[length-1]; v=path[length-2]; length-=2;
					w=matchedWithV[v]; // match v with u instead of v with w
					matchedWithU[u]=v; matchedWithV[v]=u;
				}
			}
		}
	   nbMatched=0;
	   for (u=0; u<sizeOfU; u++) if (matchedWithU[u] >= 0) nbMatched++;
	}
	return nbMatched;
}


int updateMatching_lazy(
				   int sizeOfU, // number of elements in U
				   int sizeOfV, // number of elements in V
				   Tdomain* D, // for each element u of U, D represents the elements of V compatible with u at a given depth in the search tree
				   int prof, // depth in the search tree 
				   int*  matchedWithU, // matchedWithU[i] = element of V matched with the ith element of U
				   int maxSize
){
   // try to compute a matching, matchedWithU, whose size is greater than maxSize+1 (and stops as soon as it is achieved) if there exists any  
   // return the size of the matching   

	int nbMatched = 0; // number of elements of U that are matched to a different element of V
	int matchedWithV[sizeOfV];
	int nbPred[sizeOfV]; // nbPred[i] = nb of predecessors of the ith element of V in the DAG
	int pred[sizeOfV][sizeOfU]; // pred[i][j] = jth predecessor the ith element of V in the DAG
	int nbSucc[sizeOfU]; // nbSucc[i] = nb of successors of the ith element of U in the DAG
	int succ[sizeOfU][sizeOfV]; // succ[i][j] = jth successor of the ith element of U in the DAG
	int listV[sizeOfV], listU[sizeOfU], listDV[sizeOfV], listDU[sizeOfU];
	int nbV, nbU, nbDV, nbDU;
	int i,j,k,stop,u,v,w;
	int markedV[sizeOfV], markedU[sizeOfU];
	// markedX[i]=white if X[i] is not in the DAG
	// markedX[i]=grey if X[i] has been added to the DAG, but not its successors
	// markedX[i]=black if X[i] and its successors have been added to the DAG
	// markedX[i]=toBeDeleted if X[i] must be deleted from the DAG
	// markedX[i]=deleted if X[i] has been deleted from the DAG
	
	
	memset(matchedWithV,-1,sizeOfV*sizeof(int));
	for (u=0; u<sizeOfU; u++)
		if (matchedWithU[u] >= 0){
			nbMatched++;
			matchedWithV[matchedWithU[u]]=u;
		}	
		
	if(nbMatched > maxSize+1) return nbMatched;
		
	for (u=0; u<sizeOfU; u++){
		if (matchedWithU[u]<0){
			for (i=D->firstVal[u]; i<D->firstVal[u]+D->nbVal_prof[prof][u]; i++)
				if (matchedWithV[D->val[i]]<0){// match u with v
					v=D->val[i];
					matchedWithU[u]=v; matchedWithV[v]=u; nbMatched++;
					if(nbMatched > maxSize+1) return nbMatched;
					break;
				}
		}
	}
	
	// Find augmenting paths in the DAG 
	while (nbMatched <= maxSize+1){ 
		// step 1 : build the DAG
		memset(markedU,white,sizeOfU*sizeof(int));
		memset(nbSucc,0,sizeOfU*sizeof(int));
		memset(markedV,white,sizeOfV*sizeof(int));
		memset(nbPred,0,sizeOfV*sizeof(int));
		//first layer of the DAG from the free nodes of U
		nbV=0;
		for (u=0; u<sizeOfU; u++){
			if (matchedWithU[u]==-1){// u is a free node of U
				markedU[u]=black;
				for (i=D->firstVal[u]; i<D->firstVal[u]+D->nbVal_prof[prof][u]; i++){// add edge (u,v) to the DAG
					v=D->val[i];
					pred[v][nbPred[v]++]=u;
					succ[u][nbSucc[u]++]=v;
					if (markedV[v]==white){// first time v is added to the DAG
						markedV[v]=grey; listV[nbV++]=v;
					}
				} 
			}
		}
		stop=0;
		while ((stop==0) && (nbV>0)){
			// build next layer from nodes of V to nodes of U
			nbU=0;
			for (i=0; i<nbV; i++){
				v=listV[i];
				markedV[v]=black;
				u=matchedWithV[v]; 
				if (markedU[u]==white){// edge (v,u) belongs to the DAG
					markedU[u]=grey; listU[nbU++]=u;
				}
			}
			// build next layer from nodes of U to nodes of V
			nbV=0;
			for (j=0; j<nbU; j++){
				u=listU[j];
				markedU[u]=black;
				if(D->current_sol[u]==-1)
				{
				  for (i=D->firstVal[u];i<D->firstVal[u]+D->nbVal_prof[prof][u];i++){
					v=D->val[i]; 
					if (markedV[v]!=black){// add edge (u,v) to the DAG
						pred[v][nbPred[v]++]=u;
						succ[u][nbSucc[u]++]=v;
						if (markedV[v]==white){// first time v is added to the DAG
							markedV[v]=grey; listV[nbV++]=v;
						}
						if (matchedWithV[v]==-1) stop=1; // we have found a free node !
					}
				  }
			     	}
			       else if (D->current_sol[u]>=0)
			         {
			           	v=D->current_sol[u]; 
					if (markedV[v]!=black){// add edge (u,v) to the DAG
						pred[v][nbPred[v]++]=u;
						succ[u][nbSucc[u]++]=v;
						if (markedV[v]==white){// first time v is added to the DAG
							markedV[v]=grey; listV[nbV++]=v;
						}
						if (matchedWithV[v]==-1) stop=1; // we have found a free node !
					}
				 }
			}
		}
	
	
		if(nbV==0) return nbMatched;
		
		// step 2: look for augmenting paths
		for (k=0; k<nbV; k++){ 
			v=listV[k];
			if ((matchedWithV[v]==-1) && (nbPred[v]>0)){// v is the final node of an augmenting path
				int path[sizeOfU+sizeOfV];
				int length = 1;
				path[0]=v;
				nbDV=0; nbDU=0;
				addToDelete(v,listDV,&nbDV,markedV);
				do{
					u=pred[v][0]; // (u,v) belongs to the augmenting path
					path[length++]=u;
					addToDelete(u,listDU,&nbDU,markedU);
					if (matchedWithU[u]!=-1){// u is not the initial node of the augmenting path
						v=matchedWithU[u]; // (v,u) belongs to the augmenting path
						path[length++]=v;
						addToDelete(v,listDV,&nbDV,markedV);
					}
				} while (matchedWithU[u]!=-1);
				
				// delete nodes of listDV and listDU
				while ((nbDV>0) || (nbDU>0)){
					while (nbDV>0){ // delete v
						v=listDV[--nbDV]; markedV[v]=deleted;
						u=matchedWithV[v];
						if (u!=-1) addToDelete(u,listDU,&nbDU,markedU);
						for (i=0; i<nbPred[v]; i++){
							u=pred[v][i]; // delete edge (u,v)
							for (j=0; ((j<nbSucc[u]) && (v!=succ[u][j])); j++);
							succ[u][j]=succ[u][--nbSucc[u]];
							if (nbSucc[u]==0) addToDelete(u,listDU,&nbDU,markedU);
						}
					}
					while (nbDU>0){// delete u
						u = listDU[--nbDU]; markedU[u]=deleted;
						v=matchedWithU[u];
						if (v!=-1) addToDelete(v,listDV,&nbDV,markedV);
						j=0;
						for (i=0; i<nbSucc[u]; i++){// delete edge (u,v)
							v=succ[u][i];
							for (j=0; ((j<nbPred[v]) && (u!=pred[v][j])); j++);
							pred[v][j]=pred[v][--nbPred[v]];
							if (nbPred[v]==0) addToDelete(v,listDV,&nbDV,markedV);
						}
					}
				}
				
				// Update the matching wrt the augmenting path
				while (length>1){
					u=path[length-1]; v=path[length-2]; length-=2;
					w=matchedWithV[v]; // match v with u instead of v with w
					matchedWithU[u]=v; matchedWithV[v]=u;
				}
			}
		}
	   nbMatched=0;
	   for (u=0; u<sizeOfU; u++) if (matchedWithU[u] >= 0) nbMatched++;
	}
	return nbMatched;
}
