#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>

#include "domain.h"
#include "graph.h"
#include "heuristics.h"
#include "mcis_solver.h"
#include "mccis_solvers.h"
#include "tools.h"


double nbNodes=1;      // number of nodes in the search tree
double nbFiltered=0;   // number of domain reductions
double nbFilteredSoftAllDiff=0; // number of domain reductions due to softAllDiff
double nbFilteredCCGP=0; // number of domain reductions due to connected components constraint on Gp
double nbFilteredCCGT=0; // number of domain reductions due to connected components constraint on Gt
double nbSol=0;        // number of solutions
clock_t tClock;   // starting time





void compatible(Tdomain* D, Tgraph Gp, Tgraph Gt, int vflib)
// store in D the compatibilities between vertices of Gp and Gt
{
    int u1, v1, u2, v2, i, j;

    D->compat = malloc(sizeof(char*) * D->valSize);
    for(v1 = 0; v1 < D->valSize; v1++)
    {
        D->compat[v1] = malloc(sizeof(char) * D->valSize);
        memset(D->compat[v1], 0, sizeof(char) * D->valSize);
    }
    
    for(u1 = 0; u1 < Gp.nbVertices; u1++)
    {
        for(i = D->firstVal[u1]; i < D->firstVal[u1] + D->nbVal_prof[0][u1]; i++)
        {
            v1 = D->val[i];
            for(u2 = u1 + 1; u2 < Gp.nbVertices; u2++)
            {
                for(j = D->firstVal[u2]; j < D->firstVal[u2] + D->nbVal_prof[0][u2]; j++)
                {
                    v2 = D->val[j];
                    if(v1 != v2)
                    {
                        if(Gt.isEdge[v1][v2] == Gp.isEdge[u1][u2]
                                && Gt.isEdge[v2][v1] == Gp.isEdge[u2][u1]
                                && (vflib == 0 || (Gt.edge_attr[v1][v2] == Gp.edge_attr[u1][u2] && Gt.edge_attr[v2][v1] == Gp.edge_attr[u2][u1])))
                        {
                            D->compat[D->posInComp[u1][v1]][D->posInComp[u2][v2]] = 1;
                            D->compat[D->posInComp[u2][v2]][D->posInComp[u1][v1]] = 1;
                        }
                    }
                } /* /for v2 */
            } /* /for u2 */
        } /* /for v1 */
    } /* /for u1 */
}


int node_attr_feasible(Tgraph Gp, unsigned short p, Tgraph Gt, unsigned short t)
// return true if the vertex p in Gp and the vertex t in Gt have the same label; false otherwise
{
 return (Gp.node_attr[p] == Gt.node_attr[t]);
}

int edge_attr_feasible(Tgraph Gp, unsigned short p1, unsigned short p2, Tgraph Gt, unsigned short t1, unsigned short t2)
// return true if the edge {p1,p2} in Gp and the edge {t1,t2} in Gt have the same label; false otherwise
{
 return (Gp.edge_attr[p1][p2] == Gt.edge_attr[t1][t2]);
}



void initialize(Tdomain* D, Tgraph Gp, Tgraph Gt, int vflib)
// initialize the structure D with compatibilities between the graphs Gp and Gt
{
    int u, v, maxdom = 0;

    D->globalMatching = (int*) malloc(sizeof(int) * Gp.nbVertices);

    D->nbVal_prof = malloc(sizeof(unsigned short*) * Gp.nbVertices);

    D->firstVal = malloc(sizeof(unsigned short) * Gp.nbVertices);
    D->posInVal = malloc(sizeof(unsigned short*) * Gp.nbVertices);
    D->posInComp = malloc(sizeof(unsigned short*) * Gp.nbVertices);
    D->solution = (int*) malloc(sizeof(int) * Gp.nbVertices);
    D->current_sol = (int*) malloc(sizeof(int) * Gp.nbVertices);
    
    D->nbDomincludingVal_prof = malloc(sizeof(int*) * Gp.nbVertices);

    unsigned short val[Gp.nbVertices * Gt.nbVertices];
    D->valSize = 0;

    for(u = 0; u < Gp.nbVertices; u++)
    {
        D->globalMatching[u] = -1;
        D->solution[u] = -1;
        D->current_sol[u] = -1;
        D->nbVal_prof[u] = malloc(sizeof(unsigned short) * Gp.nbVertices);
        D->posInVal[u] = malloc(sizeof(unsigned short) * Gt.nbVertices);
        D->posInComp[u] = malloc(sizeof(unsigned short) * Gt.nbVertices);
        D->nbDomincludingVal_prof[u] = calloc(Gp.nbVertices, sizeof(int));
        D->firstVal[u] = D->valSize;
        D->nbVal_prof[0][u] = 0;
        for(v = 0; v < Gt.nbVertices; v++)
        {
            D->posInVal[u][v] = Gp.nbVertices * Gt.nbVertices;
            D->posInComp[u][v] = Gp.nbVertices * Gt.nbVertices;
            if(vflib == 1)
            {
                if(node_attr_feasible(Gp, u, Gt, v) && Gp.isEdge[u][u] == Gt.isEdge[v][v] && Gp.edge_attr[u][u] == Gt.edge_attr[v][v])
                {
                    val[D->valSize] = v;
                    D->nbVal_prof[0][u]++;
                    D->posInVal[u][v] = D->valSize++;
                    D->posInComp[u][v] = D->posInVal[u][v];
                    D->nbDomincludingVal_prof[0][v]++;
                }
            }
            else if(Gp.isEdge[u][u] == Gt.isEdge[v][v])
            {
                val[D->valSize] = v;
                D->nbVal_prof[0][u]++;
                D->posInVal[u][v] = D->valSize++;
                D->posInComp[u][v] = D->posInVal[u][v];
                D->nbDomincludingVal_prof[0][v]++;
            }
        }
        if(maxdom < D->nbVal_prof[0][u]) maxdom = D->nbVal_prof[0][u];
    }

    printf("maxdom : %d\n", maxdom);
    printf("D->valSize : %d\n", D->valSize);

    D->val = malloc(sizeof(unsigned short) * D->valSize);

    for(u = 0; u < D->valSize; u++) D->val[u] = val[u];

    compatible(D, Gp, Gt, vflib);
}



void uninitialize(Tdomain* D, Tgraph Gp, Tgraph Gt, int vflib)
// free D
{
 int u;
 
   
 for (u=0; u<Gp.nbVertices; u++) 
   {
      	free(D->posInVal[u]);
      	free(D->posInComp[u]);
	free(D->nbVal_prof[u]);
	free(Gp.succ[u]);
	free(Gp.isEdge[u]);
	if(vflib==1) free(Gp.edge_attr[u]);
	free(D->nbDomincludingVal_prof[u]);
   }
   
 for (u=0; u<Gt.nbVertices; u++) 
   {
	free(Gt.succ[u]);
	free(Gt.isEdge[u]);
	if(vflib==1) free(Gt.edge_attr[u]);
   }
   
 for(u=0; u< D->valSize; u++) free(D->compat[u]);

 free(D->compat);

 free(D->posInVal);
 free(D->posInComp);
 free(D->val);
 free(D->firstVal);
 free(D->solution);
 free(D->current_sol);
 free(D->nbVal_prof);
 free(D->globalMatching);
 free(D->nbDomincludingVal_prof);
 
 
 free(Gp.succ);
 free(Gp.nbSucc);
 free(Gp.isEdge);
 free(Gt.succ);
 free(Gt.nbSucc);
 free(Gt.isEdge);
 
 if(vflib==1)
  {
 	free(Gp.node_attr);
 	free(Gp.edge_attr);
 	free(Gt.node_attr);
 	free(Gt.edge_attr);
  }
}
	
	

void display_matching_sol(Tdomain D, Tgraph Gp)
// display the matching of vertices in Gp with those in Gt stored D.solution 
{
 int i;
 
 printf("\n\nDisplay solution \n\n");
 for(i=0; i<Gp.nbVertices; i++)
   {
   	if(D.solution[i]>=0) 
   		printf("%d --> %d\n", i, D.solution[i]);
   }
}


void write_in_a_file_graph_sol(Tdomain D, Tgraph Gp, int maxsize, char* fileName)
// store in fileName the graph induced by D.solution (common induced subgraph)
{
 int i,j,deg,nb=0;
 
 FILE* fd;
 
 for(i=0; i<Gp.nbVertices; i++)
   {
   	if(D.solution[i]>=0) nb++;
   }
 
 if(nb != maxsize)
  {
    printf("ERROR: The number of matched vertices (%d) is different from the number of vertices of the maximum common subgraph (%d)", nb, maxsize); 
    exit(1);
  }
 
 if ((fd=fopen(fileName, "w"))==NULL)
  {
    printf("ERROR: Cannot open ascii input file %s", fileName); 
    exit(1);	
  }
 
 fprintf(fd,"%d\n",maxsize);
 
 for(i=0; i<Gp.nbVertices; i++)
   {
   	if(D.solution[i]>=0) 
   	  {
   	    deg = 0;
   	    for(j=0; j<Gp.nbVertices; j++)
   	      {
   	        if(D.solution[j]>=0 && Gp.isEdge[i][j] == 1) 
   	          deg++;
   	      }
   	    
   	    fprintf(fd,"%d",deg);
   	    
   	    for(j=0; j<Gp.nbVertices; j++)
   	      {
   	        if(D.solution[j]>=0 && Gp.isEdge[i][j] == 1) 
   	          fprintf(fd," %d",j);
   	      }
   	    
   	    fprintf(fd,"\n");
   	  }
   }
  
 fclose(fd);
}
 






int parse(char *softAllDiff, char * propagate, char *vflib, int * vflib_perc_lab, char *label, char *dir, char* connected,
    char *fileNameGp, char *fileNameGt, float * tlimit, char *fileNameSolution, char *display_matching, char *argv[], int argc)
{
    char ch;
    extern char *optarg;
    while((ch = getopt(argc, argv, "a:v:g:l:d:n:p:t:c:m:e:f:y:h")) != -1)
    {
        switch(ch)
        {
        case 'a':
            *softAllDiff = atoi(optarg);
            break;
        case 'e':
            *propagate = atoi(optarg);
            break;
        case 'v':
            *vflib = atoi(optarg);
            break;
        case 'g':
            *vflib_perc_lab = atoi(optarg);
            break;
	case 'y':
            *display_matching = atoi(optarg);
            break;
        case 'l':
            *label = atoi(optarg);
            break;
        case 'd':
            *dir = atoi(optarg);
            break;
        case 'n':
            *connected = atoi(optarg);
            break;
        case 'p':
            strncpy(fileNameGp, optarg, 254);
            fileNameGp[253] = '\0';
            break;
        case 't':
            strncpy(fileNameGt, optarg, 254);
            fileNameGt[253] = '\0';
            break;
        case 'm':
            *tlimit = atoi(optarg);
            break;
        case 'f':
            strncpy(fileNameSolution, optarg, 254);
            fileNameGp[253] = '\0';
            break;
        case 'h':
        case '?':
        default:
            printf("\nUsage :"
                    "\n\t-a\t1 combining with SoftAllDiff (default); 0 otherwise."
                    "\n\t-e\tpropagation levels : 0 fc, 1 mac (default), 2 gac."
                    "\n\t-v\t1 vflib (default); 0 otherwise."
                    "\n\t-g\tvflib percentage labels (default : 33)."
                    "\n\t-l\t1 labeled (default); 0 otherwise."
                    "\n\t-d\t1 directed (default); 0 otherwise."
                    "\n\t-n\t0 mcis (default); 1 branching for mccis; 2 filtering for mccis; 3 branching and filtering for mccis."
                    "\n\t-p\tInput pattern graph filename."
                    "\n\t-t\tInput target graph filename."
                    "\n\t-f\tWrite the solution graph in this file."
		    "\n\t-y\t1 display the matching solution; 0 (default) otherwise."
                    "\n\t-m\ttime limit in seconds (default 0 : no time limit)."
                    "\n\t-h\tPrint this help.\n\n");
            return(1);
            break;
        }
    }
    return(0);
}



int main(int argc, char* argv[])
{
    tClock = clock(); // initialize CPU time
    char optimal = 1; // 1 if the optimality is proven and 0 otherwise 
    // Parameters
    char fileNameGp[1024] = "mcs10_b03_s40.A00"; // pattern graph file 
    char fileNameGt[1024] = "mcs10_b03_s40.B00"; // target graph file 
    char fileNameSol[1024] = ""; // common subgraph file
    char GSAD = 1; // 1 combines with a global SoftAllDiff
    char vflib = 1; // 1 if pattern and target graph files uses vflib format
    int vflib_perc_lab = 33; // Percentage used to label graphs in vflib
    char dir = 1; // 1 if the graphs are directed and 0 otherwise
    char label = 1; // 1 if the graphs are labelled
    char propagate = 1; //propagation level (values : 0(fc),1(mac),2(gac))
    float tlimit=0; // time limit
    int maxSize; // size of common subgraph (number of vertices)  
    char connected = 0; //0-> mcis, 1->mccis by branching, 2->mccis by filtering, 3->mccis by branching and filtering
    char display_matching = 0; // 1 to display the matching
    
    if(parse(&GSAD, &propagate, &vflib, &vflib_perc_lab, &label, &dir, &connected, fileNameGp, fileNameGt, &tlimit, fileNameSol, &display_matching, argv, argc) == 1)
        return 1;
    printf("Parameters: GSAD=%d vflib=%d vflib_percentage_labels=%d dir=%d connected=%d label=%d propagate=%d fileNameGp=%s fileNameGt=%s fileNameSol=%s display_matching=%d timelimit=%f\n", GSAD, vflib, vflib_perc_lab, dir, connected, label, propagate, fileNameGp, fileNameGt, fileNameSol, display_matching, tlimit);

    Tgraph Gp; // Pattern graph
    Tgraph Gt; // Target graph
    Tgraph *p_p, *p_t;
    
    if(connected >= 1 && dir == 1)
      {
    	printf("mccis is only computed on undirected graphs\n");
    	dir = 0;
    	printf("Parameters: GSAD=%d vflib=%d vflib_percentage_labels=%d dir=%d connected=%d label=%d propagate=%d fileNameGp=%s fileNameGt=%s fileNameSol=%s display_matching=%d timelimit=%f\n", GSAD, vflib, vflib_perc_lab, dir, connected, label, propagate, fileNameGp, fileNameGt, fileNameSol, display_matching, tlimit);
      }

    printf("Reading graphs...\n");

    if(vflib) /* graph in vflib format */
    {
        if(label) /* labelled graph */
        {
            if(dir) /* directed graph */
            {
                if(createGraph_vflib(fileNameGp, &Gp, vflib_perc_lab) || createGraph_vflib(fileNameGt, &Gt, vflib_perc_lab))
                {
                    printf("Cannot read the graph\n");
                    return 1;
                }
            }
            else /* undirected graph */
            {
                if(createGraph_vflib_undir(fileNameGp, &Gp, vflib_perc_lab) || createGraph_vflib_undir(fileNameGt, &Gt, vflib_perc_lab))
                {
                    printf("Cannot read the graph\n");
                    return 1;
                }
            }
        }
        else /* unlabelled graph */
        {
            if(dir) /* directed graph */
            {
                if(createGraph_vflib_unlab(fileNameGp, &Gp) || createGraph_vflib_unlab(fileNameGt, &Gt))
                {
                    printf("Cannot read the graph\n");
                    return 1;
                }
            }
            else /* undirected graph */
            {
                if(createGraph_vflib_undir_unlab(fileNameGp, &Gp) || createGraph_vflib_undir_unlab(fileNameGt, &Gt))
                {
                    printf("Cannot read the graph\n");
                    return 1;
                }
            }
        }
    }
    else /* Graph format : number of nodes on the first line. */
    	 /* for each node, give the degree of the node followed by the list of its adjacent nodes */
         /* nodes must be numbered from 0. */
    {
        if(dir) /* directed graph */
        {
            if(createGraph(fileNameGp, &Gp) || createGraph(fileNameGt, &Gt))
            {
                printf("Cannot read the graph\n");
                return 1;
            }
        }
        else /* undirected graph */
        {
            if(createGraph_undir(fileNameGp, &Gp) || createGraph_undir(fileNameGt, &Gt))
            {
                printf("Cannot read the graph\n");
                return 1;
            }
        }
    }

    Tdomain D; // Domains

    printf("Initializing...\n");
    
    if(Gp.nbVertices <= Gt.nbVertices)
     {
      p_p = &Gp;
      p_t = &Gt;
     }
    else
     {
       p_p = &Gt;
       p_t = &Gp;
     }

	initialize(&D, *p_p, *p_t, vflib); // memory allocation and initialization of domains

	printf("Solving...\n");
	
	if(connected == 0)
	  {
		maxSize = solve_mcis(GSAD, propagate, &D, p_p, p_t, tlimit, &optimal);
		if(maxSize > 0 && verif_sol_mcis(D, *p_p, *p_t, vflib) == 0)
		{
		    printf("\n\nIncorrect solution\n\n");
		}
		else if(fileNameSol[0] != '\0')
		{
		 	write_in_a_file_graph_sol(D, Gp, maxSize, fileNameSol);
		} 
	  }
	else if (connected == 1)
	  {
		maxSize = solve_mccis_branching(GSAD, propagate, &D, p_p, p_t, tlimit, &optimal);
		if(maxSize > 0 && verif_sol_mccis(D, *p_p, *p_t, vflib) == 0)
		{
		    printf("\n\nIncorrect solution\n\n");
		}
		else if(fileNameSol[0] != '\0')
		{
		 	write_in_a_file_graph_sol(D, Gp, maxSize, fileNameSol);
		} 
	  }
	else if (connected == 2)
	  {
		maxSize = solve_mccis_filtering(GSAD, propagate, &D, p_p, p_t, tlimit, &optimal);
		if(maxSize > 0 && verif_sol_mccis(D, *p_p, *p_t, vflib) == 0)
		{
		    printf("\n\nIncorrect solution\n\n");
		}
		else if(fileNameSol[0] != '\0')
		{
		 	write_in_a_file_graph_sol(D, Gp, maxSize, fileNameSol);
		} 
	  }
	else if (connected == 3)
	  {
		maxSize = solve_mccis_branchingandfiltering(GSAD, propagate, &D, p_p, p_t, tlimit, &optimal);
		if(maxSize > 0 && verif_sol_mccis(D, *p_p, *p_t, vflib) == 0)
		{
		    printf("\n\nIncorrect solution\n\n");
		}
		else if(fileNameSol[0] != '\0')
		{
		 	write_in_a_file_graph_sol(D, Gp, maxSize, fileNameSol);
		} 
	  }
	else
	  {
	    printf("value of connected should be between 0 and 3\n");
	    exit(1);
	  }

	
	
	if(! connected)
	{
	if(optimal == 1)
		printf("\nsize optimal mcis: %d\n", maxSize);
	else
		printf("\nsize not proved optimal mcis: %d\n", maxSize);
	}
	else
	{
	if(optimal == 1)
		printf("\nsize optimal mccis: %d\n", maxSize);
	else
		printf("\nsize not proved optimal mccis: %d\n", maxSize);
	}
	
	printf("nodes : %lf\nfiltered : %lf", nbNodes, nbFiltered);
	if(propagate == 2)
		printf("\nGAC-GSAD filtered : %lf", nbFilteredSoftAllDiff);
	
	if(connected >= 2)	
		printf("\nCCGP filtered : %lf\nCCGT filtered : %lf", nbFilteredCCGP, nbFilteredCCGT);

	printf("\ntime : %f\n", (float)(clock() - tClock) / CLOCKS_PER_SEC);

	if(display_matching) display_matching_sol(D, *p_p);
	
	uninitialize(&D, *p_p, *p_t, vflib);

    return 0;
}

