#include <time.h>
#include <sys/time.h>

#include "mccis_solvers.h"
#include "values.h"
#include "heuristics.h"
#include "tools.h"


int verif_sol_mccis(Tdomain D, Tgraph Gp, Tgraph Gt, int vflib)
// return 1 if the matching in D.solution is a connected common induced subgraph of Gp and Gt, 0 otherwise
{
 int i,j;
 
 char inccx[Gt.nbVertices];
 char inccx2[Gp.nbVertices];
 
 for (i=0; i<Gt.nbVertices; i++)
	inccx[i] = 0;

					
 
 for(i=0; i<Gp.nbVertices; i++)
   {
   	inccx2[i] = 0;
   	if(D.solution[i]>=0)
   	  {
   		inccx[D.solution[i]] = 1;
   		inccx2[i] = 1;
   		if (vflib==1 && Gp.node_attr[i]!=Gt.node_attr[D.solution[i]])
   	  		      	{
   	  		       		printf("\nError on node_attribute\n");
   	  		      		return 0;
   	  		      	}
   	  		      	
   		for(j=0; j<Gp.nbVertices; j++)
   	  	  {
   	  		if(i!=j && D.solution[j]>=0)
   	  		  {
   	  		    if(D.solution[i]==D.solution[j]) 
   	  		      {
   	  		        printf("\nTwo different nodes matched to the same\n");
   	  		        return 0;
   	  		      }
   	  		     if (Gp.isEdge[i][j]!=Gt.isEdge[D.solution[i]][D.solution[j]] || Gp.isEdge[j][i]!=Gt.isEdge[D.solution[j]][D.solution[i]])
   	  		      	{
   	  		       		printf("\nError on induced subgraph\n");
   	  		       		printf("D.sol[%d]=%d D.sol[%d]=%d\n",i,D.solution[i],j,D.solution[j]);
   	  		      		return 0;
   	  		      	}
   	  		     if (vflib==1 && (Gp.edge_attr[i][j]!=Gt.edge_attr[D.solution[i]][D.solution[j]] || Gp.edge_attr[j][i]!=Gt.edge_attr[D.solution[j]][D.solution[i]]))
   	  		      	{
   	  		       		printf("\nError on edge_attribute\n");
   	  		      		return 0;
   	  		      	}
   	  		      	
   	  		   }
   	  	  }
   	  }
   }

   
 if(! isconnectedcomponent(&Gp, inccx2))
   {
     printf("\n Error : subgraph in Gp non connected\n");
     return 0;
   }
  
 if(! isconnectedcomponent(&Gt, inccx))
   {
     printf("\n Error : subgraph in Gt non connected\n");
     return 0;
   }
   
 return 1;
}




int solve_mccis_branching(char GSAD, // if GSAD=1 then global SoftAllDiff is called at each iteration for filtering  
			  char propagate, //propagation levels : 0 fc, 1 mac, 2 gac.
			  Tdomain* D, // D stores the domains and the compatibilities, and will store the computed solution
			  Tgraph* Gp, // pattern graph
			  Tgraph* Gt, // target graph
			  float tlimit, // time limit
			  char* optimal // optimal = 1 if the optimality of the computed solution is proven, 0 otherwise
){ 
	// compute a connected common induced subgraph of Gp and Gt within the time limit using the branching strategy
        // if optimal = 1 then the subgraph is a mccis
	// return the size of subgraph
	

	int u,v, minDom=-1, nbempty,maxSize=0,currentSize=0,prof=0,nbMatched=0,nbunmat; 
	int sol_prof[Gp->nbVertices];
	int Gp_deg[Gp->nbVertices]; // dynamic degrees
	int Gt_deg[Gt->nbVertices]; // dynamic degrees
	unsigned char consist;
	char connected[Gp->nbVertices];
	int first_choice = -1; //depth of the first variable assigned with a value != -2
		
	
	/* initializing*/
  	
  	for (u=0; u<Gp->nbVertices; u++) 
  	  {
  	  	sol_prof[u]=-1;
  	  	Gp_deg[u]=Gp->nbSucc[u];
  	  	D->current_sol[u]=-1;
  	  	connected[u] = 0;
  	  }
  	  
  	for (u=0; u<Gt->nbVertices; u++)  Gt_deg[u]=Gt->nbSucc[u];
        
	 while(prof!=-1)
	  {
    		/* choice of the next edge in Gp */
    		
    		 
    		 if(minDom==-1) 
    		  { 
    		 	if(first_choice == -1)
    		  		minDom=choose_node_mindom(D, Gp, prof);
    		  	else
    		 		minDom=choose_node_mindom_cc(D, Gp, prof, connected);
    		 	
    		 	if(minDom == -1)
    		 		{
    		 		  printf("There is no more variable to assign: minDom == -1\n");
    		 		  exit(1);
    		 		}
    		  }
    		

		 
	  	 /* choice of the value of minDom */ 

    		 v=choose_value_gmatch_mindiff(D, Gp_deg, Gt_deg, minDom, prof);


		
		D->current_sol[minDom]=v;
		
		if(D->globalMatching[minDom]>=0) nbMatched--;
		
		D->globalMatching[minDom]=v;
		
		if(v!=-2)
		 {
		   currentSize++;
		   nbMatched++;
		   if(first_choice == -1)
		   	first_choice = prof;
		 }
		
		nbNodes++;
		
		
		sol_prof[prof]=minDom;
		
		if(v!=-2) 
		  {  		
		  	/* removing v from the domain of minDom */
		  	int oldPos = D->posInVal[minDom][v];
		  	D->nbVal_prof[prof][minDom]--;
			int newPos = D->firstVal[minDom]+D->nbVal_prof[prof][minDom];
			D->val[oldPos] = D->val[newPos];
			D->val[newPos] = v;
			D->posInVal[minDom][D->val[oldPos]] = oldPos;
			D->posInVal[minDom][v] = newPos;
			
			for (u=0; u<Gt->nbSucc[v]; u++) Gt_deg[Gt->succ[v][u]]--; /* update degrees */
		  }
		  
		for (u=0; u<Gp->nbSucc[minDom]; u++) Gp_deg[Gp->succ[minDom][u]]--; /* update degrees */
		
		consist=1;
		nbempty=0;
		nbunmat=0;
		
		if(currentSize < Gp->nbVertices)
		  {
			/* initializing the domain of the next level */
			
			
			if(prof+1 < Gp->nbVertices)
			  {
				for (u=0; u<Gp->nbVertices; u++) 
			  	  {
			  		if(D->current_sol[u]==-1) 
		  	  	  	  {
		    				D->nbVal_prof[prof+1][u] = D->nbVal_prof[prof][u];
		  	  	  	  }
		  	  		else if(D->current_sol[u]==-2) D->nbVal_prof[prof+1][u] = 0;
		  	  		else D->nbVal_prof[prof+1][u] = 1;
		  	  	  }
		  	  	  
		  	  	
		  	  	  
		  	  	/* update current connected component */
		  	  	 
		  	  	 if(D->current_sol[minDom] >= 0)
				 {
				  for (u=0; u<Gp->nbSucc[minDom]; u++) 
			  	   {
		  	  	  	  connected[Gp->succ[minDom][u]] = 1;
		  	  	   }
		  	  	 }

		  	  	 
		  	  	 

		 	
				// match minDom to v and filter the unmatched domains
			
			
				if(v!=-2) 
				  {
				     	matchVertices(prof+1,minDom,v,D,Gp,Gt,&nbMatched);
				  }
				
				if (GSAD>0)
				{
				   if(nbMatched <= maxSize)
				   {
				   if(propagate == 2)
				        nbMatched=filter_GAC(prof+1, D, Gp, Gt, maxSize);
				   else
				     	nbMatched=filter(prof+1, D, Gp, Gt, maxSize);
				   }	
				}
				else nbMatched=maxSize+2;
				
				if(nbMatched>maxSize)
				{
				 nbempty=0;
				 nbunmat=0;
				 for(u=0; u<Gp->nbVertices; u++) 
				  {
				    if(D->current_sol[u]==-1 && D->nbVal_prof[prof+1][u]==0) nbempty++;
				    else if(D->current_sol[u]==-2) nbunmat++;
				  }
				 
				 if(propagate >= 1)
				 {
				 if(Gp->nbVertices-nbempty-nbunmat == maxSize+1)  
				    {
				     consist=AC3(D,Gp,prof+1);
				     if (GSAD>0)
				      {
				     	if(propagate == 2)
				        	nbMatched=filter_GAC(prof+1, D, Gp, Gt, maxSize);
				        else
				     		nbMatched=filter(prof+1, D, Gp, Gt, maxSize);
				      }
				     nbempty=0;
				     for(u=0; u<Gp->nbVertices; u++) if(D->current_sol[u]==-1 && D->nbVal_prof[prof+1][u]==0) nbempty++;
				    }
				 }
				}
			   }
			   
		        
		        int nbConnected_toassign = 0;
			
			for (u=0; u<Gp->nbVertices; u++) 
			  if(D->current_sol[u]==-1 && D->nbVal_prof[prof+1][u] > 0 && connected[u])
		  	  	  nbConnected_toassign++;
		    
			minDom=-1;
			
			
			if ((nbempty==Gp->nbVertices-(prof+1)) || (Gp->nbVertices-nbempty-nbunmat <= maxSize) || (nbMatched <= maxSize) || !consist || (nbConnected_toassign == 0 && first_choice >= 0))
		  	  {	/* all remaining domains are empty or empty domain number ensures no improvement  after matchVertex  or 
		  	  	   remaining variables are not included in the current connected component
		  	  	   prof+1-currentSize is the number of unmatched (v==-2) */
		  		  	
				if(currentSize > maxSize) 
		  	  	  {
					for (u=0; u<Gp->nbVertices; u++) D->solution[u]=D->current_sol[u];
					maxSize = currentSize;
					printf("currentSize : %d <-- %f s\n", currentSize, (float)(clock() - tClock) / CLOCKS_PER_SEC);
		  	  	  }
				while(prof>-1 && D->current_sol[sol_prof[prof]]==-2) 
				/* backtrack to a level with a no empty domain */
			  	  {
			  		for (u=0; u<Gp->nbSucc[sol_prof[prof]]; u++) Gp_deg[Gp->succ[sol_prof[prof]][u]]++; /* update degrees */
			  		D->current_sol[sol_prof[prof]]=-1;  
			  		D->globalMatching[sol_prof[prof]]=-1;
			  		prof--; 
			  	  }
				if(prof>-1)
			  	  {
			  		currentSize--;
			  		minDom=sol_prof[prof];
			  		v=D->current_sol[minDom];
			  		
			  		/* update degrees */
			  		for (u=0; u<Gp->nbSucc[minDom]; u++) Gp_deg[Gp->succ[minDom][u]]++;
			  		for (u=0; u<Gt->nbSucc[v]; u++) Gt_deg[Gt->succ[v][u]]++;
			  
			  	  	D->current_sol[minDom]=-1;
			  	  	D->globalMatching[minDom]=-1;
			  	  	nbMatched--;
			  	  	
			  	  	if(prof <= first_choice)
				          first_choice = -1;
				          
				        /* update current connected component */
			  	  	
			  	  	for (u=0; u<Gp->nbVertices; u++) 
			  	  		connected[u] = 0;
				 
					 for (u=0; u < prof; u++) 
					 {
					   if(D->current_sol[sol_prof[u]] >= 0)
					    {
					     connected[sol_prof[u]] = 1;
					     for (v=0; v < Gp->nbSucc[sol_prof[u]]; v++) 
				  	  		connected[Gp->succ[sol_prof[u]][v]] = 1;
				  	    }
				  	 }	
			  	  	
			  	  }
		  	  }
			else prof++;
		  }
		else
		  {
			for (u=0; u<Gp->nbVertices; u++) D->solution[u]=D->current_sol[u];
			maxSize = currentSize;
			printf("currentSize : %d <-- %f s\n", currentSize, (float)(clock() - tClock) / CLOCKS_PER_SEC);
			prof=-1;
		  }
		  
		if(tlimit > 0)
		{
        	if(((float)(clock()-tClock)/CLOCKS_PER_SEC) >= tlimit) 
        	 {
        	   printf("\nTIMEOUT\n");
        	   *optimal = 0;
        	   prof=-1;
        	 }
        	}
	  }

 	return maxSize;
}


int solve_mccis_filtering(char GSAD, // if GSAD=1 then global SoftAllDiff is called at each iteration for filtering  
			  char propagate, //propagation levels : 0 fc, 1 mac, 2 gac.
			  Tdomain* D, // D stores the domains and the compatibilities, and will store the computed solution
			  Tgraph* Gp, // pattern graph
			  Tgraph* Gt, // target graph
			  float tlimit, // time limit
			  char* optimal // optimal = 1 if the optimality of the computed solution is proven, 0 otherwise
){ 
	// compute a connected common induced subgraph of Gp and Gt within the time limit using the filtering strategy
        // if optimal = 1 then the subgraph is a mccis
	// return the size of subgraph
	

	int u,v, minDom=-1, nbempty,maxSize=0,currentSize=0,prof=0,nbMatched=0,nbunmat; 
	int sol_prof[Gp->nbVertices];
	int Gp_deg[Gp->nbVertices]; // dynamic degrees
	int Gt_deg[Gt->nbVertices]; // dynamic degrees
	unsigned char consist;
	int inccGp[Gp->nbVertices];
	int outccGp[Gp->nbVertices];
	int inccGt[Gt->nbVertices];
	int outccGt[Gt->nbVertices];
	int nbInccGp, nbInccGt, nbOutccGp, nbOutccGt;
	int first_choice = -1; //depth of the first variable assigned with a value != -2
	Tgraph Gpcopy; 
        Tgraph Gtcopy;
        
        char modifGtcopy, modifGpcopy;
        
        int xremovedfromGpcopy[Gp->nbVertices][2*Gp->nbEdges];
        int nb_xremovedfromGpcopy[Gp->nbVertices];
        
        int xremovedfromGtcopy[Gp->nbVertices][2*Gt->nbEdges];
        int nb_xremovedfromGtcopy[Gp->nbVertices];
        
        int start=-1, stop=-1;
        
        char disconnect = 0;
        
        char inccx[Gt->nbVertices];
        char inccx2[Gp->nbVertices];
        
        
        copy_graph(Gp, &Gpcopy);
        copy_graph(Gt, &Gtcopy);

		
	
	/* initializing*/
  	
  	for (u=0; u<Gp->nbVertices; u++) 
  	  {
  	  	sol_prof[u]=-1;
  	  	Gp_deg[u]=Gp->nbSucc[u];
  	  	D->current_sol[u]=-1;
  	  	nb_xremovedfromGpcopy[u] = 0;
  	  	nb_xremovedfromGtcopy[u] = 0;
  	  }
  	  
  	for (u=0; u<Gt->nbVertices; u++)  
  	  {
  	    Gt_deg[u]=Gt->nbSucc[u];
  	    inccGt[u] = u;
  	  }
  	nbInccGt = Gt->nbVertices;
  	nbOutccGt = 0;
        
	while(prof!=-1)
	  {
    		
    		/* choice of the next edge in Gp */
    		
    		
    		disconnect = 0;
    		 
    		 if(minDom==-1) 
    		  { 
    		 	minDom=choose_node_mindom(D, Gp, prof);
    		 	
    		 	if(minDom == -1)
    		 		{
    		 		  printf("There is no more variable to assign: minDom == -1\n");
    		 		  exit(1);
    		 		}
    		  }
    		

		 
	  	/* choice of the value of minDom */ 

    		v=choose_value_gmatch_mindiff(D, Gp_deg, Gt_deg, minDom, prof);

		
		D->current_sol[minDom]=v;
		
		
		if(D->globalMatching[minDom]>=0) nbMatched--;
		
		D->globalMatching[minDom]=v;
		
		if(v!=-2)
		 {
		   currentSize++;
		   nbMatched++;
		   if(first_choice == -1)
		   	first_choice = prof;
		 }
		else
		 {
		   remove_edgesofx(&Gpcopy, minDom, xremovedfromGpcopy[prof], &(nb_xremovedfromGpcopy[prof]));  
		 }
		
		nbNodes++;
		
		
		sol_prof[prof]=minDom;
		
		if(v!=-2) 
		  {  		
		  	/* removing v from the domain of minDom */
		  	int oldPos = D->posInVal[minDom][v];
		  	D->nbVal_prof[prof][minDom]--;
			int newPos = D->firstVal[minDom]+D->nbVal_prof[prof][minDom];
			D->val[oldPos] = D->val[newPos];
			D->val[newPos] = v;
			D->posInVal[minDom][D->val[oldPos]] = oldPos;
			D->posInVal[minDom][v] = newPos;
			
			for (u=0; u<Gt->nbSucc[v]; u++) Gt_deg[Gt->succ[v][u]]--; /* update degrees */
		  }
		  
		for (u=0; u<Gp->nbSucc[minDom]; u++) Gp_deg[Gp->succ[minDom][u]]--; /* update degrees */
		
		consist=1;
		nbempty=0;
		nbunmat=0;
		
		if(currentSize < Gp->nbVertices)
		  {

			if(prof+1 < Gp->nbVertices)
			  {
				/* initializing the domain of the next level */
				
				start=-1;
				
				for (u=0; u<Gp->nbVertices; u++) 
			  	  {
			  		if(D->current_sol[u]==-1) 
		  	  	  	  {
		    				D->nbVal_prof[prof+1][u] = D->nbVal_prof[prof][u];
		    				if(D->nbVal_prof[prof+1][u] == 0 && Gpcopy.nbSucc[u] > 0)
		    				  {
		    					remove_edgesofx(&Gpcopy, u, xremovedfromGpcopy[prof], &(nb_xremovedfromGpcopy[prof]));
		    				  }
		    				
		    				if(start == -1)
		    					start = u;
		    				stop = u+1;
		  	  	  	  }
		  	  		else if(D->current_sol[u]==-2) D->nbVal_prof[prof+1][u] = 0;
		  	  		else D->nbVal_prof[prof+1][u] = 1;
		  	  	  }
		  	       
		  	  	  
		  	  	  for (u=0; u<Gt->nbVertices; u++) 
		  	  		D->nbDomincludingVal_prof[prof+1][u] = D->nbDomincludingVal_prof[prof][u];
			  	  
			  	  int v1;
			  		
			  	  /* update the number of domains containing values of minDom except its current value */
			  	  	
		  		  for(v1 = D->firstVal[minDom]; v1 < D->firstVal[minDom]+D->nbVal_prof[prof][minDom]; v1++)
		  		   {
		  		     D->nbDomincludingVal_prof[prof+1][D->val[v1]]--;
		  		   }
		  	  	  
		  	  	/* filtering all values of variables that cannot be in the current connected component of Gp */
				    
		  	  	if(nb_xremovedfromGpcopy[prof] > 0 && first_choice >= 0)
		  	  	  {
		  	  	    connectedcomponentincludingx (&Gpcopy, sol_prof[first_choice], inccGp, &nbInccGp, outccGp, &nbOutccGp); 
		  	  	    
		  	  	    for (u=0; u<nbOutccGp; u++) 
				  	  {
				  		int u1 = outccGp[u];
				  		if(D->current_sol[u1] >= 0)
				  			disconnect = 1;
				  		else if(D->current_sol[u1]==-1 && D->nbVal_prof[prof+1][u1] > 0) 
				  		  {
			    				int v1;
			    				nbFilteredCCGP += D->nbVal_prof[prof+1][u1];
			    				for (v1=D->firstVal[u1]; v1<D->firstVal[u1]+D->nbVal_prof[prof+1][u1]; v1++) 
			    					D->nbDomincludingVal_prof[prof+1][D->val[v1]]--;
			    				D->nbVal_prof[prof+1][u1] = 0;
			    				
			    				if(D->globalMatching[u1]>=0) nbMatched--;
			    				
			    			  }
			  	  	  }
		  	  	  }
		  	  	  

			        if(! disconnect)
			        {
				if(v!=-2) 
				  {
				     	/* match minDom to v and filter the unmatched domains */
				     	
				     	matchVertices(prof+1,minDom,v,D,Gp,Gt,&nbMatched);
				  }

				
				/* remove from Gtcopy edges incident to nodes (corresponding to values removed from all domains) that cannot be no more in the
				   current connected component */
				
				modifGtcopy = 0;
				
				for (v=0; v<nbInccGt; v++)  
			  	  { 
				      int v1 = inccGt[v];
				      
				      if(D->nbDomincludingVal_prof[prof+1][v1] == 0 && Gtcopy.nbSucc[v1] > 0)
						{
						  remove_edgesofx(&Gtcopy, v1, xremovedfromGtcopy[prof], &(nb_xremovedfromGtcopy[prof]));
						}
				  }

			  	 if(nb_xremovedfromGtcopy[prof] > 0 && first_choice >= 0)
			  	  {
			  	    /* update the current connected component in GTcopy and remove from domains values corresponding to nodes 
			  	       that are no longer in this component */
				    
			  	    connectedcomponentincludingx (&Gtcopy, D->current_sol[sol_prof[first_choice]], inccGt, &nbInccGt, outccGt, &nbOutccGt); 
			  	    for (v=0; v<nbOutccGt; v++)  
			  	    {
			  	      int v1 = outccGt[v];
			  	      for (u=start; u<stop; u++) 
				  	   {
				  	      if(D->current_sol[u]==-1 && isInD_prof(u, v1, D, prof+1)) 
				  		  {
				  		    if (last_pos2(u,v1,prof+1,D)) nbMatched--;
				  		    nbFilteredCCGT++;
				  		    modifGtcopy = 1;
				  		  }
				  	   }
			  	    }
			  	  }
			  	  
			        /* filtering until fixpoint */ 
			        while(modifGtcopy && !disconnect)
			         {
			           modifGtcopy = 0;
			           modifGpcopy = 0;
			           for (u=start; u<stop; u++) 
			  	    {

			  		if(D->current_sol[u]==-1 && D->nbVal_prof[prof + 1][u] == 0 && Gpcopy.nbSucc[u] > 0) 
		  	  	  	  {
		    				remove_edgesofx(&Gpcopy, u, xremovedfromGpcopy[prof], &(nb_xremovedfromGpcopy[prof]));
		    				modifGpcopy = 1;
		  	  	  	  }
		  	  	    }
		  	  	   
		  	  	   if(modifGpcopy)
		  	  	   {
		  	  	    modifGpcopy = 0;
		  	  	    
		  	  	    connectedcomponentincludingx (&Gpcopy, sol_prof[first_choice], inccGp, &nbInccGp, outccGp, &nbOutccGp);  	    
		  	  	    
		  	  	    for (u=0; u<nbOutccGp; u++) 
				  	  {
				  		int u1 = outccGp[u];
				  		if(D->current_sol[u1] >= 0)
				  			disconnect = 1;
				  		else if(D->current_sol[u1]==-1 && D->nbVal_prof[prof+1][u1] > 0) 
				  		  {
			    				int v1;
			    				nbFilteredCCGP += D->nbVal_prof[prof+1][u1];
			    				for (v1=D->firstVal[u1]; v1<D->firstVal[u1]+D->nbVal_prof[prof+1][u1]; v1++) 
			    					D->nbDomincludingVal_prof[prof+1][D->val[v1]]--;
			    				D->nbVal_prof[prof+1][u1] = 0;
			    				
			    				if(D->globalMatching[u1]>=0) nbMatched--;
			    				
			    				modifGpcopy = 1;
			    				
			    			  }
			  	  	  }
		  	  	    }
		  	  	   
		  	  	   if(modifGpcopy)
		  	  	    { 
		  	  	      for (v=0; v<nbInccGt; v++)  
				  	  { 
					      int v1 = inccGt[v];
					      if(D->nbDomincludingVal_prof[prof+1][v1] == 0 && Gtcopy.nbSucc[v1] > 0)
							{
							  remove_edgesofx(&Gtcopy, v1, xremovedfromGtcopy[prof], &(nb_xremovedfromGtcopy[prof]));
							  modifGtcopy = 1;
							}
					  }

				      if(modifGtcopy)
				  	  {
				  	    modifGtcopy = 0;
				  	    
				  	    /* update the current connected component in GTcopy and remove from domains values corresponding to nodes 
				  	       that are no longer in this component */
 
				  	    connectedcomponentincludingx (&Gtcopy, D->current_sol[sol_prof[first_choice]], inccGt, &nbInccGt, outccGt, &nbOutccGt);
				  	    
				  	    for (v=0; v<nbOutccGt; v++)  
				  	    {
				  	      int v1 = outccGt[v];
				  	      for (u=start; u<stop; u++) 
					  	   {
					  	      if(D->current_sol[u]==-1 && isInD_prof(u, v1, D, prof+1)) 
					  		  {
					  		    if (last_pos2(u,v1,prof+1,D)) nbMatched--;
					  		    nbFilteredCCGT++;
					  		    modifGtcopy = 1;
					  		  }
					  	   }
				  	    }
				  	  }
		  	  	    }
		  	  	 }

				if(! disconnect)
				{				
				/* compute the number of variables that can be assigned different values */
				if (GSAD>0)
				{
				   if(nbMatched <= maxSize)
				    {	
				       if(propagate == 2)
				        	nbMatched=filter_GAC(prof+1, D, Gp, Gt, maxSize);
				       else
				     		nbMatched=filter(prof+1, D, Gp, Gt, maxSize);
				    }
				}
				else nbMatched=maxSize+2;
				
				if(nbMatched>maxSize)
				{
								 
				 nbempty=0;
				 nbunmat=0;
				 for(u=0; u<Gp->nbVertices; u++) 
				  {
				    if(D->current_sol[u]==-1 && D->nbVal_prof[prof+1][u]==0) nbempty++;
				    else if(D->current_sol[u]==-2) nbunmat++;
				  }
				 
				 if(propagate >= 1)
				 {
				 if(Gp->nbVertices-nbempty-nbunmat == maxSize+1)  
				    {
				     /* the value -2 can be removed from domains since all remaining variables need to be assigned with a value != -2 
				        to improve the current best solution : therefore AC can be run on domains */
				     consist=AC3(D,Gp,prof+1);
				     if (GSAD>0)
				      {
				     	if(propagate == 2)
				        	nbMatched=filter_GAC(prof+1, D, Gp, Gt, maxSize);
				        else
				     		nbMatched=filter(prof+1, D, Gp, Gt, maxSize);
				      }
				     nbempty=0;
				     for(u=start; u<stop; u++) if(D->current_sol[u]==-1 && D->nbVal_prof[prof+1][u]==0) nbempty++;
				    }
				  }
				}
			       }
			     }

			   }
			   
		    
			minDom=-1;

			
			if ((nbempty==Gp->nbVertices-(prof+1)) || (Gp->nbVertices-nbempty-nbunmat <= maxSize) || (nbMatched <= maxSize) || !consist || disconnect)
		  	  {	/* all remaining domains are empty or empty domain number ensures no improvement after matchVertex  or 
		  	  	   the current graph is disconnected
		  	  	   prof+1-currentSize is the number of unmatched (v==-2) */
  	
				if(disconnect)
				{
				 if(currentSize - 1 > maxSize) 
		  	  	  {
					for (u=0; u<Gt->nbVertices; u++)
						inccx[u] = 0;	
					 
					 for(u=0; u<Gp->nbVertices; u++)
					   {
					   	inccx2[u] = 0;
					   	if(D->current_sol[u]>=0 && u != sol_prof[prof])
					   	  {
					   		inccx[D->current_sol[u]] = 1;
					   		inccx2[u] = 1;
					   	  }
					   }
					   
					if(isconnectedcomponent_filtering(Gp, inccx2) && isconnectedcomponent_filtering(Gt, inccx))
					{
					 for (u=0; u<Gp->nbVertices; u++) D->solution[u]=D->current_sol[u];
					 D->solution[sol_prof[prof]] = -2;
					 maxSize = currentSize - 1;
					 printf("currentSize : %d <-- %f s\n", currentSize - 1, (float)(clock() - tClock) / CLOCKS_PER_SEC);
					}	
		  	  	  }
		  	  	 }
		  	  	else
		  	  	 {
		  	  	   if(currentSize > maxSize) 
		  	  	    {
					for (u=0; u<Gt->nbVertices; u++)
						inccx[u] = 0;	
					 
					 for(u=0; u<Gp->nbVertices; u++)
					   {
					   	inccx2[u] = 0;
					   	if(D->current_sol[u]>=0)
					   	  {
					   		inccx[D->current_sol[u]] = 1;
					   		inccx2[u] = 1;
					   	  }
					   }
					   
					if(isconnectedcomponent_filtering(Gp, inccx2) && isconnectedcomponent_filtering(Gt, inccx))
					{
					 for (u=0; u<Gp->nbVertices; u++) D->solution[u]=D->current_sol[u];
					 maxSize = currentSize;
					 printf("currentSize : %d <-- %f s\n", currentSize, (float)(clock() - tClock) / CLOCKS_PER_SEC);
					}
		  	  	    }
		  	  	 }
		  	  	  
		  	  	modifGtcopy = 0;
		  	  	
				while(prof>-1 && D->current_sol[sol_prof[prof]]==-2) 
				/* backtrack to a level with a no empty domain */
			  	  {
			  		
			  		for (u=0; u<Gp->nbSucc[sol_prof[prof]]; u++) Gp_deg[Gp->succ[sol_prof[prof]][u]]++; /* update degrees */
			  		D->current_sol[sol_prof[prof]]=-1;  
			  		D->globalMatching[sol_prof[prof]]=-1;
			  		
			  		/* update graph copies */
			  		for (u = 0; u < nb_xremovedfromGpcopy[prof]; u+=2)
			  		  {
			  			restore_edgesofx(&Gpcopy, xremovedfromGpcopy[prof][u], xremovedfromGpcopy[prof][u+1]);
			  		  }
			  		nb_xremovedfromGpcopy[prof] = 0;
			  		
			  		for (u = 0; u < nb_xremovedfromGtcopy[prof]; u+=2)
			  		  {
			  			restore_edgesofx(&Gtcopy, xremovedfromGtcopy[prof][u], xremovedfromGtcopy[prof][u+1]);
			  		  }
			  		  
			  		if(nb_xremovedfromGtcopy[prof] > 0)  
			  			modifGtcopy = 1;
			  		nb_xremovedfromGtcopy[prof] = 0;
			  		
			  		prof--; 
			  	  }
				if(prof>-1)
			  	  {
			  	
			  		currentSize--;
			  		minDom=sol_prof[prof];
			  		v=D->current_sol[minDom];
			  		D->nbDomincludingVal_prof[prof][v]--;
			  		
			  		/* update degrees */
			  		for (u=0; u<Gp->nbSucc[minDom]; u++) Gp_deg[Gp->succ[minDom][u]]++;
			  		for (u=0; u<Gt->nbSucc[v]; u++) Gt_deg[Gt->succ[v][u]]++;
			  
			  	  	D->current_sol[minDom]=-1;
			  	  	D->globalMatching[minDom]=-1;
			  	  	nbMatched--;
			  	  	
			  	  	if(prof <= first_choice)
				          first_choice = -1;
	
				  	 
				        /* update graph copies */

  					for (u = 0; u < nb_xremovedfromGpcopy[prof]; u+=2)
			  		  {
			  			restore_edgesofx(&Gpcopy, xremovedfromGpcopy[prof][u], xremovedfromGpcopy[prof][u+1]);
			  		  }
			  		nb_xremovedfromGpcopy[prof] = 0;
			  		
			  		for (u = 0; u < nb_xremovedfromGtcopy[prof]; u+=2)
			  		  {
			  			restore_edgesofx(&Gtcopy, xremovedfromGtcopy[prof][u], xremovedfromGtcopy[prof][u+1]);
			  		  }
			  		if(nb_xremovedfromGtcopy[prof] > 0)  
			  			modifGtcopy = 1;
			  		nb_xremovedfromGtcopy[prof] = 0;
			  		
			  		if(modifGtcopy)
			  		 {
			  		   for (v=0; v < Gt->nbVertices; v++) 
						inccGt[v] = v;
					   nbInccGt = Gt->nbVertices;
					   nbOutccGt = 0;
			  		 }
        
			  	  }
		  	  }
			else prof++;
		  }
		else
		  {
			for (u=0; u<Gp->nbVertices; u++) D->solution[u]=D->current_sol[u];
			maxSize = currentSize;
			printf("currentSize : %d <-- %f s\n", currentSize, (float)(clock() - tClock) / CLOCKS_PER_SEC);
			prof=-1;
		  }
		
		if(tlimit > 0)
		{
        	if(((float)(clock()-tClock)/CLOCKS_PER_SEC) >= tlimit) 
        	 {
        	   printf("\nTIMEOUT\n");
        	   *optimal = 0;
        	   prof=-1;
        	 }
        	}
	  }
	  
	deleteGraphcopy(&Gpcopy);
	deleteGraphcopy(&Gtcopy);

 	return maxSize;
}



int solve_mccis_branchingandfiltering(char GSAD, // if GSAD=1 then global SoftAllDiff is called at each iteration for filtering  
			  char propagate, //propagation levels : 0 fc, 1 mac, 2 gac.
			  Tdomain* D, // D stores the domains and the compatibilities, and will store the computed solution
			  Tgraph* Gp, // pattern graph
			  Tgraph* Gt, // target graph
			  float tlimit, // time limit
			  char* optimal // optimal = 1 if the optimality of the computed solution is proven, 0 otherwise
){ 
	// compute a connected common induced subgraph of Gp and Gt within the time limit using both the branching and filtering strategies
        // if optimal = 1 then the subgraph is a mccis
	// return the size of subgraph
	

	int u,v, minDom=-1, nbempty,maxSize=0,currentSize=0,prof=0,nbMatched=0,nbunmat; 
	int sol_prof[Gp->nbVertices];
	int Gp_deg[Gp->nbVertices]; // dynamic degrees
	int Gt_deg[Gt->nbVertices]; // dynamic degrees
	unsigned char consist;
	char connected[Gp->nbVertices];
	int inccGp[Gp->nbVertices];
	int outccGp[Gp->nbVertices];
	int inccGt[Gt->nbVertices];
	int outccGt[Gt->nbVertices];
	int nbInccGp, nbInccGt, nbOutccGp, nbOutccGt;
	int first_choice = -1; //depth of the first variable assigned with a value != -2
	Tgraph Gpcopy; 
        Tgraph Gtcopy;
        
        char modifGtcopy, modifGpcopy;
        
        int xremovedfromGpcopy[Gp->nbVertices][2*Gp->nbEdges];
        int nb_xremovedfromGpcopy[Gp->nbVertices];
        
        int xremovedfromGtcopy[Gp->nbVertices][2*Gt->nbEdges];
        int nb_xremovedfromGtcopy[Gp->nbVertices];
        
        int start=-1, stop=-1;
        
        
        copy_graph(Gp, &Gpcopy);
        copy_graph(Gt, &Gtcopy);

		
	
	/* initializing*/
  	
  	for (u=0; u<Gp->nbVertices; u++) 
  	  {
  	  	sol_prof[u]=-1;
  	  	Gp_deg[u]=Gp->nbSucc[u];
  	  	D->current_sol[u]=-1;
  	  	nb_xremovedfromGpcopy[u] = 0;
  	  	nb_xremovedfromGtcopy[u] = 0;
  	  	connected[u] = 0;
  	  }
  	  
  	for (u=0; u<Gt->nbVertices; u++)  
  	  {
  	    Gt_deg[u]=Gt->nbSucc[u];
  	    inccGt[u] = u;
  	  }
  	nbInccGt = Gt->nbVertices;
  	nbOutccGt = 0;
        
	while(prof!=-1)
	  {
    		
    		/* choice of the next edge in Gp */
    		
    		 
    		 if(minDom==-1) 
    		  { 
    		 	if(first_choice == -1)
    		  		minDom=choose_node_mindom(D, Gp, prof);
    		  	else
    		 		minDom=choose_node_mindom_cc(D, Gp, prof, connected);
    		 	
    		 	if(minDom == -1)
    		 		{
    		 		  printf("There is no more variable to assign: minDom == -1\n");
    		 		  exit(1);
    		 		}
    		  }
    		

		 
	  	 /* choice of the value of minDom */ 

    		 v=choose_value_gmatch_mindiff(D, Gp_deg, Gt_deg, minDom, prof);

		
		D->current_sol[minDom]=v;
		
		if(D->globalMatching[minDom]>=0) nbMatched--;
		
		D->globalMatching[minDom]=v;
		
		if(v!=-2)
		 {
		   currentSize++;
		   nbMatched++;
		   if(first_choice == -1)
		   	first_choice = prof;
		 }
		else
		 {
		   remove_edgesofx(&Gpcopy, minDom, xremovedfromGpcopy[prof], &(nb_xremovedfromGpcopy[prof]));  
		 }
		
		nbNodes++;
		
		
		sol_prof[prof]=minDom;
		
		if(v!=-2) 
		  {  		
		  	/* removing v from the domain of minDom */
		  	int oldPos = D->posInVal[minDom][v];
		  	D->nbVal_prof[prof][minDom]--;
			int newPos = D->firstVal[minDom]+D->nbVal_prof[prof][minDom];
			D->val[oldPos] = D->val[newPos];
			D->val[newPos] = v;
			D->posInVal[minDom][D->val[oldPos]] = oldPos;
			D->posInVal[minDom][v] = newPos;
			
			for (u=0; u<Gt->nbSucc[v]; u++) Gt_deg[Gt->succ[v][u]]--; /* update degrees */
		  }
		  
		for (u=0; u<Gp->nbSucc[minDom]; u++) Gp_deg[Gp->succ[minDom][u]]--; /* update degrees */
		
		consist=1;
		nbempty=0;
		nbunmat=0;
		
		if(currentSize < Gp->nbVertices)
		  {

			if(prof+1 < Gp->nbVertices)
			  {
				/* initializing the domain of the next level */
				
				start=-1;
				
				for (u=0; u<Gp->nbVertices; u++) 
			  	  {
			  		if(D->current_sol[u]==-1) 
		  	  	  	  {
		    				D->nbVal_prof[prof+1][u] = D->nbVal_prof[prof][u];
		    				if(D->nbVal_prof[prof+1][u] == 0 && Gpcopy.nbSucc[u] > 0)
		    				  {
		    					remove_edgesofx(&Gpcopy, u, xremovedfromGpcopy[prof], &(nb_xremovedfromGpcopy[prof]));
		    				  }
		    				
		    				if(start == -1)
		    					start = u;
		    				stop = u+1;
		  	  	  	  }
		  	  		else if(D->current_sol[u]==-2) D->nbVal_prof[prof+1][u] = 0;
		  	  		else D->nbVal_prof[prof+1][u] = 1;
		  	  	  }
		  	       
		  	  	  
		  	  	  for (u=0; u<Gt->nbVertices; u++) 
		  	  		D->nbDomincludingVal_prof[prof+1][u] = D->nbDomincludingVal_prof[prof][u];
			  	  
			  	  int v1;
			  		
			  	  /* update the number of domains containing values of minDom except its current value */
			  	  	
		  		  for(v1 = D->firstVal[minDom]; v1 < D->firstVal[minDom]+D->nbVal_prof[prof][minDom]; v1++)
		  		   {
		  		     D->nbDomincludingVal_prof[prof+1][D->val[v1]]--;
		  		   }
		  	  	  
		  	  	/* filtering all values of variables that cannot be in the current connected component of Gp */
				    
		  	  	if(nb_xremovedfromGpcopy[prof] > 0 && first_choice >= 0)
		  	  	  {
		  	  	    connectedcomponentincludingx (&Gpcopy, sol_prof[first_choice], inccGp, &nbInccGp, outccGp, &nbOutccGp); 
		  	  	    
		  	  	    for (u=0; u<nbOutccGp; u++) 
				  	  {
				  		int u1 = outccGp[u];
				  		if(D->current_sol[u1]==-1 && D->nbVal_prof[prof+1][u1] > 0) 
				  		  {
			    				int v1;
			    				nbFilteredCCGP += D->nbVal_prof[prof+1][u1];
			    				for (v1=D->firstVal[u1]; v1<D->firstVal[u1]+D->nbVal_prof[prof+1][u1]; v1++) 
			    					D->nbDomincludingVal_prof[prof+1][D->val[v1]]--;
			    				D->nbVal_prof[prof+1][u1] = 0;
			    				
			    				if(D->globalMatching[u1]>=0) nbMatched--;
			    				
			    			  }
			  	  	  }
		  	  	  }
		  	  	  
		  	  	/* update current connected component */
		  	  	 
		  	  	 if(D->current_sol[minDom] >= 0)
				 {
				  for (u=0; u<Gpcopy.nbSucc[minDom]; u++) 
			  	   {
		  	  	  	  connected[Gpcopy.succ[minDom][u]] = 1;
		  	  	   }
		  	  	 }

			
				if(v!=-2) 
				  {
				     	/* match minDom to v and filter the unmatched domains */
				     	
				     	matchVertices(prof+1,minDom,v,D,Gp,Gt,&nbMatched);
				  }

				
				/* remove from Gtcopy edges incident to nodes (corresponding to values removed from all domains) that cannot be no more in the
				   current connected component */
				
				modifGtcopy = 0;
				
				for (v=0; v<nbInccGt; v++)  
			  	  { 
				      int v1 = inccGt[v];
				      
				      if(D->nbDomincludingVal_prof[prof+1][v1] == 0 && Gtcopy.nbSucc[v1] > 0)
						{
						  remove_edgesofx(&Gtcopy, v1, xremovedfromGtcopy[prof], &(nb_xremovedfromGtcopy[prof]));
						}
				  }

			  	 if(nb_xremovedfromGtcopy[prof] > 0 && first_choice >= 0)
			  	  {
			  	    /* update the current connected component in GTcopy and remove from domains values corresponding to nodes 
			  	       that are no longer in this component */
				    
			  	    connectedcomponentincludingx (&Gtcopy, D->current_sol[sol_prof[first_choice]], inccGt, &nbInccGt, outccGt, &nbOutccGt); 
			  	    for (v=0; v<nbOutccGt; v++)  
			  	    {
			  	      int v1 = outccGt[v];
			  	      for (u=start; u<stop; u++) 
				  	   {
				  	      if(D->current_sol[u]==-1 && isInD_prof(u, v1, D, prof+1)) 
				  		  {
				  		    if (last_pos2(u,v1,prof+1,D)) nbMatched--;
				  		    nbFilteredCCGT++;
				  		    modifGtcopy = 1;
				  		  }
				  	   }
			  	    }
			  	  }
			  	  
			        /* filtering until fixpoint */ 
			        while(modifGtcopy)
			         {
			           modifGtcopy = 0;
			           modifGpcopy = 0;
			           for (u=start; u<stop; u++) 
			  	    {

			  		if(D->current_sol[u]==-1 && D->nbVal_prof[prof + 1][u] == 0 && Gpcopy.nbSucc[u] > 0) 
		  	  	  	  {
		    				remove_edgesofx(&Gpcopy, u, xremovedfromGpcopy[prof], &(nb_xremovedfromGpcopy[prof]));
		    				modifGpcopy = 1;
		  	  	  	  }
		  	  	    }
		  	  	   
		  	  	   if(modifGpcopy)
		  	  	   {
		  	  	    modifGpcopy = 0;
		  	  	    
		  	  	    connectedcomponentincludingx (&Gpcopy, sol_prof[first_choice], inccGp, &nbInccGp, outccGp, &nbOutccGp);  	    
		  	  	    
		  	  	    for (u=0; u<nbOutccGp; u++) 
				  	  {
				  		int u1 = outccGp[u];
				  		if(D->current_sol[u1]==-1 && D->nbVal_prof[prof+1][u1] > 0) 
				  		  {
			    				int v1;
			    				nbFilteredCCGP += D->nbVal_prof[prof+1][u1];
			    				for (v1=D->firstVal[u1]; v1<D->firstVal[u1]+D->nbVal_prof[prof+1][u1]; v1++) 
			    					D->nbDomincludingVal_prof[prof+1][D->val[v1]]--;
			    				D->nbVal_prof[prof+1][u1] = 0;
			    				
			    				if(D->globalMatching[u1]>=0) nbMatched--;
			    				
			    				modifGpcopy = 1;
			    				
			    			  }
			  	  	  }
		  	  	    }
		  	  	   
		  	  	   if(modifGpcopy)
		  	  	    { 
		  	  	      for (v=0; v<nbInccGt; v++)  
				  	  { 
					      int v1 = inccGt[v];
					      if(D->nbDomincludingVal_prof[prof+1][v1] == 0 && Gtcopy.nbSucc[v1] > 0)
							{
							  remove_edgesofx(&Gtcopy, v1, xremovedfromGtcopy[prof], &(nb_xremovedfromGtcopy[prof]));
							  modifGtcopy = 1;
							}
					  }

				      if(modifGtcopy)
				  	  {
				  	    modifGtcopy = 0;
				  	    
				  	    /* update the current connected component in GTcopy and remove from domains values corresponding to nodes 
				  	       that are no longer in this component */
 
				  	    connectedcomponentincludingx (&Gtcopy, D->current_sol[sol_prof[first_choice]], inccGt, &nbInccGt, outccGt, &nbOutccGt);
				  	    
				  	    for (v=0; v<nbOutccGt; v++)  
				  	    {
				  	      int v1 = outccGt[v];
				  	      for (u=start; u<stop; u++) 
					  	   {
					  	      if(D->current_sol[u]==-1 && isInD_prof(u, v1, D, prof+1)) 
					  		  {
					  		    if (last_pos2(u,v1,prof+1,D)) nbMatched--;
					  		    nbFilteredCCGT++;
					  		    modifGtcopy = 1;
					  		  }
					  	   }
				  	    }
				  	  }
		  	  	    }
		  	  	 }

								
				/* compute the number of variables that can be assigned different values */
				if (GSAD>0)
				{
				   if(nbMatched <= maxSize)
				    {	
				       if(propagate == 2)
				        	nbMatched=filter_GAC(prof+1, D, Gp, Gt, maxSize);
				       else
				     		nbMatched=filter(prof+1, D, Gp, Gt, maxSize);
				    }	
				}
				else nbMatched=maxSize+2;
				
				if(nbMatched>maxSize)
				{
								 
				 nbempty=0;
				 nbunmat=0;
				 for(u=0; u<Gp->nbVertices; u++) 
				  {
				    if(D->current_sol[u]==-1 && D->nbVal_prof[prof+1][u]==0) nbempty++;
				    else if(D->current_sol[u]==-2) nbunmat++;
				  }
				  
				 
				 if(propagate >= 1)
				 {
				 if(Gp->nbVertices-nbempty-nbunmat == maxSize+1)  
				    {
				     /* the variable -2 can be removed from domains since all remaining variables need to be assigned with a value != -2 
				        to improve the current best solution : therefore AC can be run on domains */
				     consist=AC3(D,Gp,prof+1);
				     if (GSAD>0)
				      {
				     	if(propagate == 2)
				        	nbMatched=filter_GAC(prof+1, D, Gp, Gt, maxSize);
				        else
				     		nbMatched=filter(prof+1, D, Gp, Gt, maxSize);
				      }
				     nbempty=0;
				     for(u=start; u<stop; u++) if(D->current_sol[u]==-1 && D->nbVal_prof[prof+1][u]==0) nbempty++;
				    }
				 }
				}

			   }
			   
		    
			minDom=-1;
			
			int nbConnected_toassign = 0;
			
			for (u=0; u<Gp->nbVertices; u++) 
			  if(D->current_sol[u]==-1 && D->nbVal_prof[prof+1][u] > 0 && connected[u])
		  	  	  	  nbConnected_toassign++;

			
			if ((nbempty==Gp->nbVertices-(prof+1)) || (Gp->nbVertices-nbempty-nbunmat <= maxSize) || (nbMatched <= maxSize) || !consist || (nbConnected_toassign == 0 && first_choice >= 0))
		  	  {	/* all remaining domains are empty or empty domain number ensures no improvement  after matchVertex  or 
		  	  	   remaining variables are not included in the current connected component
		  	  	   prof+1-currentSize is the number of unmatched (v==-2) */
  	
				
				if(currentSize > maxSize) 
		  	  	  {
					for (u=0; u<Gp->nbVertices; u++) D->solution[u]=D->current_sol[u];
					maxSize = currentSize;
					printf("currentSize : %d <-- %f s\n", currentSize, (float)(clock() - tClock) / CLOCKS_PER_SEC);
		  	  	  }
		  	  	  
		  	  	modifGtcopy = 0;
		  	  	
				while(prof>-1 && D->current_sol[sol_prof[prof]]==-2) 
				/* backtrack to a level with a no empty domain */
			  	  {
			  		
			  		for (u=0; u<Gp->nbSucc[sol_prof[prof]]; u++) Gp_deg[Gp->succ[sol_prof[prof]][u]]++; /* update degrees */
			  		D->current_sol[sol_prof[prof]]=-1;  
			  		D->globalMatching[sol_prof[prof]]=-1;
			  		
			  		/* update graph copies */
			  		for (u = 0; u < nb_xremovedfromGpcopy[prof]; u+=2)
			  		  {
			  			restore_edgesofx(&Gpcopy, xremovedfromGpcopy[prof][u], xremovedfromGpcopy[prof][u+1]);
			  		  }
			  		nb_xremovedfromGpcopy[prof] = 0;
			  		
			  		for (u = 0; u < nb_xremovedfromGtcopy[prof]; u+=2)
			  		  {
			  			restore_edgesofx(&Gtcopy, xremovedfromGtcopy[prof][u], xremovedfromGtcopy[prof][u+1]);
			  		  }
			  		  
			  		if(nb_xremovedfromGtcopy[prof] > 0)  
			  			modifGtcopy = 1;
			  		nb_xremovedfromGtcopy[prof] = 0;
			  		
			  		prof--; 
			  	  }
				if(prof>-1)
			  	  {
			  	
			  		currentSize--;
			  		minDom=sol_prof[prof];
			  		v=D->current_sol[minDom];
			  		D->nbDomincludingVal_prof[prof][v]--;
			  		
			  		/* update degrees */
			  		for (u=0; u<Gp->nbSucc[minDom]; u++) Gp_deg[Gp->succ[minDom][u]]++;
			  		for (u=0; u<Gt->nbSucc[v]; u++) Gt_deg[Gt->succ[v][u]]++;
			  
			  	  	D->current_sol[minDom]=-1;
			  	  	D->globalMatching[minDom]=-1;
			  	  	nbMatched--;
			  	  	
			  	  	/* update current connected component */
			  	  	
			  	  	for (u=0; u<Gp->nbVertices; u++) 
			  	  		connected[u] = 0;
				 
					 for (u=0; u < prof; u++) 
					 {
					   if(D->current_sol[sol_prof[u]] >= 0)
					    {
					     connected[sol_prof[u]] = 1;
					     for (v=0; v < Gp->nbSucc[sol_prof[u]]; v++) 
				  	  		connected[Gp->succ[sol_prof[u]][v]] = 1;
				  	    }
				  	 }
			  	  	
			  	  	if(prof <= first_choice)
				          first_choice = -1;
	
				  	 
				        /* update graph copies */

  					for (u = 0; u < nb_xremovedfromGpcopy[prof]; u+=2)
			  		  {
			  			restore_edgesofx(&Gpcopy, xremovedfromGpcopy[prof][u], xremovedfromGpcopy[prof][u+1]);
			  		  }
			  		nb_xremovedfromGpcopy[prof] = 0;
			  		
			  		for (u = 0; u < nb_xremovedfromGtcopy[prof]; u+=2)
			  		  {
			  			restore_edgesofx(&Gtcopy, xremovedfromGtcopy[prof][u], xremovedfromGtcopy[prof][u+1]);
			  		  }
			  		if(nb_xremovedfromGtcopy[prof] > 0)  
			  			modifGtcopy = 1;
			  		nb_xremovedfromGtcopy[prof] = 0;
			  		
			  		if(modifGtcopy)
			  		 {
			  		   for (v=0; v < Gt->nbVertices; v++) 
						inccGt[v] = v;
					   nbInccGt = Gt->nbVertices;
					   nbOutccGt = 0;
			  		 }
        
			  	  }
		  	  }
			else prof++;
		  }
		else
		  {
			for (u=0; u<Gp->nbVertices; u++) D->solution[u]=D->current_sol[u];
			maxSize = currentSize;
			printf("currentSize : %d <-- %f s\n", currentSize, (float)(clock() - tClock) / CLOCKS_PER_SEC);
			prof=-1;
		  }
		if(tlimit > 0)
		{
        	if(((float)(clock()-tClock)/CLOCKS_PER_SEC) >= tlimit) 
        	 {
        	   printf("\nTIMEOUT\n");
        	   *optimal = 0;
        	   prof=-1;
        	 }
        	}
	  }
	  
	deleteGraphcopy(&Gpcopy);
	deleteGraphcopy(&Gtcopy);

 	return maxSize;
}

