#ifndef MCCIS_SOLVERS_H
#define MCCIS_SOLVERS_H

#include <time.h>
#include <sys/time.h>

#include "graph.h"
#include "domain.h"


int verif_sol_mccis(Tdomain D, Tgraph Gp, Tgraph Gt, int vflib);
// return 1 if the matching in D.solution is a connected common induced subgraph of Gp and Gt, 0 otherwise



int solve_mccis_branching(char GSAD, // if GSAD=1 then global SoftAllDiff is called at each iteration for filtering  
			  char propagate, //propagation levels : 0 fc, 1 mac, 2 gac.
			  Tdomain* D, // D stores the domains and the compatibilities, and will store the computed solution
			  Tgraph* Gp, // pattern graph
			  Tgraph* Gt, // target graph
			  float tlimit, // time limit
			  char* optimal // optimal = 1 if the optimality of the computed solution is proven, 0 otherwise
); 
	// compute a connected common induced subgraph of Gp and Gt within the time limit using the branching strategy
        // if optimal = 1 then the subgraph is a mccis
	// return the size of subgraph




int solve_mccis_filtering(char GSAD, // if GSAD=1 then global SoftAllDiff is called at each iteration for filtering  
			  char propagate, //propagation levels : 0 fc, 1 mac, 2 gac.
			  Tdomain* D, // D stores the domains and the compatibilities, and will store the computed solution
			  Tgraph* Gp, // pattern graph
			  Tgraph* Gt, // target graph
			  float tlimit, // time limit
			  char* optimal // optimal = 1 if the optimality of the computed solution is proven, 0 otherwise
); 
	// compute a connected common induced subgraph of Gp and Gt within the time limit using the filtering strategy
        // if optimal = 1 then the subgraph is a mccis
	// return the size of subgraph




int solve_mccis_branchingandfiltering(char GSAD, // if GSAD=1 then global SoftAllDiff is called at each iteration for filtering  
			  char propagate, //propagation levels : 0 fc, 1 mac, 2 gac.
			  Tdomain* D, // D stores the domains and the compatibilities, and will store the computed solution
			  Tgraph* Gp, // pattern graph
			  Tgraph* Gt, // target graph
			  float tlimit, // time limit
			  char* optimal // optimal = 1 if the optimality of the computed solution is proven, 0 otherwise
); 
	// compute a connected common induced subgraph of Gp and Gt within the time limit using both the branching and filtering strategies
        // if optimal = 1 then the subgraph is a mccis
	// return the size of subgraph


#endif
