#include <time.h>
#include <sys/time.h>

#include "mcis_solver.h"
#include "values.h"
#include "heuristics.h"
#include "tools.h"



int verif_sol_mcis(Tdomain D, Tgraph Gp, Tgraph Gt, int vflib)
// return 1 if the matching in D.solution is a common induced subgraph of Gp and Gt, 0 otherwise
{
 int i,j;
 
 for(i=0; i<Gp.nbVertices; i++)
   {
   	if(D.solution[i]>=0)
   	  {
   		if (vflib==1 && Gp.node_attr[i]!=Gt.node_attr[D.solution[i]])
   	  		      	{
   	  		       		printf("\nError on node_attribute\n");
   	  		      		return 0;
   	  		      	}
   	  		      	
   		for(j=0; j<Gp.nbVertices; j++)
   	  	  {
   	  		if(i!=j && D.solution[j]>=0)
   	  		  {
   	  		    if(D.solution[i]==D.solution[j]) 
   	  		      {
   	  		        printf("\nTwo different nodes matched to the same\n");
   	  		        return 0;
   	  		      }
   	  		     if (Gp.isEdge[i][j]!=Gt.isEdge[D.solution[i]][D.solution[j]] || Gp.isEdge[j][i]!=Gt.isEdge[D.solution[j]][D.solution[i]])
   	  		      	{
   	  		       		printf("\nError on induced subgraph\n");
   	  		       		printf("D.sol[%d]=%d D.sol[%d]=%d\n",i,D.solution[i],j,D.solution[j]);
   	  		      		return 0;
   	  		      	}
   	  		     if (vflib==1 && (Gp.edge_attr[i][j]!=Gt.edge_attr[D.solution[i]][D.solution[j]] || Gp.edge_attr[j][i]!=Gt.edge_attr[D.solution[j]][D.solution[i]]))
   	  		      	{
   	  		       		printf("\nError on edge_attribute\n");
   	  		      		return 0;
   	  		      	}
   	  		      	
   	  		   }
   	  	  }
   	  }
   }
   
 return 1;
}




int solve_mcis(char GSAD, // if GSAD=1 then global SoftAllDiff is called at each iteration for filtering  
			  char propagate, //propagation levels : 0 fc, 1 mac, 2 gac.
			  Tdomain* D, // D stores the domains and the compatibilities, and will store the computed solution
			  Tgraph* Gp, // pattern graph
			  Tgraph* Gt, // target graph
			  float tlimit, // time limit
			  char* optimal // optimal = 1 if the optimality of the computed solution is proven, 0 otherwise
){
	// compute a common induced subgraph of Gp and Gt within the time limit
        // if optimal = 1 then the subgraph is a mcis
	// return the size of subgraph
	

	int u,v, minDom=-1, nbempty,maxSize=0,currentSize=0,prof=0,nbMatched=0,nbunmat; 
	int sol_prof[Gp->nbVertices];
	int Gp_deg[Gp->nbVertices]; // dynamic degrees
	int Gt_deg[Gt->nbVertices]; // dynamic degrees
	unsigned char consist;
		
	
	/* initializing*/
  	
  	for (u=0; u<Gp->nbVertices; u++) 
  	  {
  	  	sol_prof[u]=-1;
  	  	Gp_deg[u]=Gp->nbSucc[u];
  	  	D->current_sol[u]=-1;
  	  }
  	  
  	for (u=0; u<Gt->nbVertices; u++)  Gt_deg[u]=Gt->nbSucc[u];

        printf("currentSize : %d <-- %f s\n",maxSize, (float)(clock() - tClock) / CLOCKS_PER_SEC);
        
	 while(prof!=-1)
	  {
    		/* choice of the next edge in Gp */
    		 if(minDom==-1) 
    		  {
    		 	minDom=choose_node_mindom(D, Gp, prof);
    		  }
    		

		 
	  	/* choice of the value of minDom */ 

    		v=choose_value_gmatch_mindiff(D, Gp_deg, Gt_deg, minDom, prof);
    		 
		
		D->current_sol[minDom]=v;
		
		if(D->globalMatching[minDom]>=0) nbMatched--;
		
		D->globalMatching[minDom]=v;
		
		if(v!=-2)
		 {
		   currentSize++;
		   nbMatched++;
		 }
		
		nbNodes++;
		
		sol_prof[prof]=minDom;
		
		if(v!=-2) 
		  {  		
		  	/* removing v from the domain of minDom */
		  	int oldPos = D->posInVal[minDom][v];
		  	D->nbVal_prof[prof][minDom]--;
			int newPos = D->firstVal[minDom]+D->nbVal_prof[prof][minDom];
			D->val[oldPos] = D->val[newPos];
			D->val[newPos] = v;
			D->posInVal[minDom][D->val[oldPos]] = oldPos;
			D->posInVal[minDom][v] = newPos;
			
			for (u=0; u<Gt->nbSucc[v]; u++) Gt_deg[Gt->succ[v][u]]--; /* update degrees */
		  }
		  
		for (u=0; u<Gp->nbSucc[minDom]; u++) Gp_deg[Gp->succ[minDom][u]]--; /* update degrees */
		
		consist=1;
		
		if(currentSize < Gp->nbVertices)
		  {
			/* initializing the domain of the next level */
			
			
			if(prof+1 < Gp->nbVertices)
			  {
				for (u=0; u<Gp->nbVertices; u++) 
			  	  {
			  		if(D->current_sol[u]==-1) 
		  	  	  	  {
		    				D->nbVal_prof[prof+1][u] = D->nbVal_prof[prof][u];
		  	  	  	  }
		  	  		else if(D->current_sol[u]==-2) D->nbVal_prof[prof+1][u] = 0;
		  	  		else D->nbVal_prof[prof+1][u] = 1;
		  	  	  }
		 	
				// match minDom to v and filter the unmatched domains
			
			
				if(v!=-2) 
				  {
				     	matchVertices(prof+1,minDom,v,D,Gp,Gt,&nbMatched);
				  }
				
				if (GSAD>0)
				{
				   if(nbMatched <= maxSize) 
				     {
				        if(propagate == 2)
				        	nbMatched=filter_GAC(prof+1, D, Gp, Gt, maxSize);
				        else
				        	nbMatched=filter(prof+1, D, Gp, Gt, maxSize);	
				        
				     }
				}
				else nbMatched=maxSize+2;
				
				if(nbMatched>maxSize)
				{
				 nbempty=0;
				 nbunmat=0;
				 for(u=0; u<Gp->nbVertices; u++) 
				  {
				    if(D->current_sol[u]==-1 && D->nbVal_prof[prof+1][u]==0) nbempty++;
				    else if(D->current_sol[u]==-2) nbunmat++;
				  }
				 
				 if(propagate >= 1)
				 {
				 if(Gp->nbVertices-nbempty-nbunmat == maxSize+1)  
				    {
				     consist=AC3(D,Gp,prof+1);
				     if (GSAD>0)
				      {
				     	if(propagate == 2)
				        	nbMatched=filter_GAC(prof+1, D, Gp, Gt, maxSize);
				        else
				     		nbMatched=filter(prof+1, D, Gp, Gt, maxSize);
				      }
				     nbempty=0;
				     for(u=0; u<Gp->nbVertices; u++) if(D->current_sol[u]==-1 && D->nbVal_prof[prof+1][u]==0) nbempty++;
				    }
				 }
				}
			   }
		    
			minDom=-1;
			
			
			if ((nbempty==Gp->nbVertices-(prof+1)) || (Gp->nbVertices-nbempty-nbunmat <= maxSize) || (nbMatched <= maxSize) || !consist)
		  	  {	// all remaining domains are empty or empty domain number ensures no improvement  after matchVertex 
		  	  	// prof+1-currentSize is the number of unmatched (v==-2)
		  		  	
				if(currentSize > maxSize) 
		  	  	  {
					for (u=0; u<Gp->nbVertices; u++) D->solution[u]=D->current_sol[u];
					maxSize = currentSize;
					printf("currentSize : %d <-- %f s\n", currentSize, (float)(clock() - tClock) / CLOCKS_PER_SEC);
		  	  	  }
				while(prof>-1 && D->current_sol[sol_prof[prof]]==-2) 
				/* backtrack to a level with a no empty domain */
			  	  {
			  		for (u=0; u<Gp->nbSucc[sol_prof[prof]]; u++) Gp_deg[Gp->succ[sol_prof[prof]][u]]++; /* update degrees */
			  		D->current_sol[sol_prof[prof]]=-1;  
			  		D->globalMatching[sol_prof[prof]]=-1;
			  		prof--; 
			  	  }
				if(prof>-1)
			  	  {
			  		currentSize--;
			  		minDom=sol_prof[prof];
			  		v=D->current_sol[minDom];
			  		
			  		/* update degrees */
			  		for (u=0; u<Gp->nbSucc[minDom]; u++) Gp_deg[Gp->succ[minDom][u]]++;
			  		for (u=0; u<Gt->nbSucc[v]; u++) Gt_deg[Gt->succ[v][u]]++;
			  
			  	  	D->current_sol[minDom]=-1;
			  	  	D->globalMatching[minDom]=-1;
			  	  	nbMatched--;
			  	  }
		  	  }
			else prof++;
		  }
		else
		  {
			for (u=0; u<Gp->nbVertices; u++) D->solution[u]=D->current_sol[u];
			maxSize = currentSize;
			printf("currentSize : %d <-- %f s\n", currentSize, (float)(clock() - tClock) / CLOCKS_PER_SEC);
			prof=-1;
		  }
		if(tlimit > 0)
		{
        	if(((float)(clock()-tClock)/CLOCKS_PER_SEC) >= tlimit) 
        	 {
        	   printf("\nTIMEOUT\n");
        	   *optimal = 0;
        	   prof=-1;
        	 }
        	}
	  }

 	return maxSize;
}


