#ifndef SOFTALLDIFF_H
#define SOFTALLDIFF_H

#include "domain.h"
#include "colors.h"

void DFS(int sizeOfU, int sizeOfV, int u, int* color, int* nbSucc, int succ[sizeOfV][sizeOfU], int* matchedWithU, int* order, int* nb);
  // perform a depth first search, starting from u, in the bipartite graph Go=(U,V,E) such that
  // U = vertices of Gp
  // V = vertices of Gt
  // E = { (u,matchedWithU[u]) / u is a vertex of Gp } U
  //     { (v,u) / v is a vertex of D[u] which is not matched to v}
  // Given a vertex v of Gt, nbSucc[v]=number of sucessors of v and succ[v]=list of successors of v
  // order[nb^out+1..nb^in] contains the vertices discovered by the DFS
  
void SCC(int sizeOfU, int sizeOfV, int* markedV, int* markedU, 
	 int* nbSucc, int succ[sizeOfV][sizeOfU], 
	 int* nbPred, int pred[sizeOfU][sizeOfV], int* matchedWithU,int* matchedWithV);
  // postrelation: markedV[v]==markedU[u] iff they belong to the same
  // strongly connected component in the bipartite graph Go=(U,V,E) such that
  // U = vertices of Gp
  // V = vertices of Gt
  // E = { (u,matchedWithU[u]) / u is a vertex of Gp } U
  //     { (v,u) / v is a vertex of D[u] which is not matched to v}
  // Given a vertex v of Gt, nbSucc[v]=number of sucessors of v and succ[v]=list of successors of v

int checkSoftAllDiff(
		 int sizeOfU, // number of elements in U
		 int sizeOfV, // number of elements in V
		 Tdomain* D, // for each element u of U, D represents the elements of V compatible with u at a given depth in the search tree
		 int prof, // depth in the search tree 
		 int* matchedWithU, // matchedWithU[i] = element of V matched with the ith element of U
		 int* nbToRemove, // (*nbToRemove) = number of values to remove from domains
		 int* toRemove, // remove toRemove[2i+1] from the domain of toRemove[2i]
		 int maxSize // best current solution : maxSize+1 elements in U must be matched to improve the current best solution
		 );
  // compute a largest matching matchedWithU
  // compute toRemove elements of V that does not appear in any largest matching, if the size of the matching is equal to maxSize+1, 
  // return the size of the matching
  
  
int checkSoftAllDiff_lazy(
		 int sizeOfU, // number of elements in U
		 int sizeOfV, // number of elements in V
		 Tdomain* D, // for each element u of U, D represents the elements of V compatible with u at a given depth in the search tree
		 int prof, // depth in the search tree 
		 int* matchedWithU, // matchedWithU[i] = element of V matched with the ith element of U
		 int* nbToRemove, // (*nbToRemove) = number of values to remove from domains
		 int* toRemove, // remove toRemove[2i+1] from the domain of toRemove[2i]
		 int maxSize // best current solution : maxSize+1 elements in U must be matched to improve the current solution 
		 );
  // try to compute a matching, matchedWithU, whose size is greater than maxSize+1 (and stops as soon as it is achieved) if there exists any  
  // compute in toRemove elements of V that does not appear in any largest matching, if the size of the matching is equal to maxSize+1 
  // return the size of the matching

#endif
