#include "tools.h"
#include "softAllDiff.h"
#include "values.h"


int isInD_prof(int u, int v, Tdomain* D, int prof){
	// return true if v belongs to the domain of u at the depth prof in the search tree; false otherwise
	return (D->posInVal[u][v]<D->firstVal[u]+D->nbVal_prof[prof][u]);
}



	
int last_pos(int u, int v, int prof, Tdomain* D, Tgraph* Gp){
	// remove v from the domain of u at the depth prof in the search tree and update the globalMatching and nbFiltered
	// return 1 if u was matched with v in the globalMatching, 0 otherwise

	D->nbDomincludingVal_prof[prof][v]--;
	
	int oldPos = D->posInVal[u][v];
	
	nbFiltered++;
	D->nbVal_prof[prof][u]--;
	int newPos = D->firstVal[u]+D->nbVal_prof[prof][u];
	

	D->val[oldPos] = D->val[newPos];
	D->val[newPos] = v;
	D->posInVal[u][D->val[oldPos]] = oldPos;
	D->posInVal[u][v] = newPos;
	
	// update globalMatching
	if (D->globalMatching[u]==v && D->current_sol[u]==-1)
	  {
	    D->globalMatching[u]=-1;
	    return 1;
	  }
	return 0;
}


int last_pos2(int u, int v, int prof, Tdomain* D){
	// remove v from the domain of u at the depth prof in the search tree and update the globalMatching
	// return 1 if u was matched with v in the globalMatching, 0 otherwise

	D->nbDomincludingVal_prof[prof][v]--;
	int oldPos = D->posInVal[u][v];
	D->nbVal_prof[prof][u]--;
	int newPos = D->firstVal[u]+D->nbVal_prof[prof][u];
	

	D->val[oldPos] = D->val[newPos];
	D->val[newPos] = v;
	D->posInVal[u][D->val[oldPos]] = oldPos;
	D->posInVal[u][v] = newPos;
	
	// update globalMatching
	if (D->globalMatching[u]==v && D->current_sol[u]==-1)
	  {
	    D->globalMatching[u]=-1;
	    return 1;
	  }
	return 0;
}




void matchVertices(int prof, int u, int v, Tdomain* D, Tgraph* Gp, Tgraph* Gt, int* nbMatched)
	// filter domains of other vertices at the depth prof in the search tree consequently of the matching of u with v and update the globalMatching
{
	int j, u2,uc=D->posInComp[u][v],jc;

	for (u2=0; u2<Gp->nbVertices; u2++)
	  {
	    	if(D->current_sol[u2]==-1 && D->nbVal_prof[prof][u2]>0)
	      	  {
			if (isInD_prof(u2,v,D,prof) && last_pos(u2,v,prof,D,Gp)) (*nbMatched)--;

			
		  	j=D->firstVal[u2]; 
			while (j<D->firstVal[u2]+D->nbVal_prof[prof][u2])
			  {
			    jc=D->posInComp[u2][D->val[j]];
			    if(D->compat[uc][jc]==0)
			     {
			       if(last_pos(u2,D->val[j],prof,D,Gp)) (*nbMatched)--;
			     }
			    else j++;
			  }
		   }
	  }
}



unsigned char AC3(Tdomain* D, Tgraph* Gp, int prof){
	// perform arc consistency and return 0 if an inconsistency is detected at the depth prof in the search tree and 1 otherwise
	int u1, u2, i, j;
	unsigned short jc;
	unsigned char turn=1;
	while (turn){
		turn=0;
		for(u1=0; u1<Gp->nbVertices;u1++)
		 {
		  if(D->current_sol[u1]==-1 && D->nbVal_prof[prof][u1]>0)
		   {
		     for(u2=0; u2<Gp->nbVertices;u2++)
		      {
		 	if(u2!=u1 && D->current_sol[u2]==-1 && D->nbVal_prof[prof][u2]>0)
		 	 {
		  	   i=D->firstVal[u1]; 
		  	   while (i<D->firstVal[u1]+D->nbVal_prof[prof][u1])
			    {
			     jc=D->posInComp[u1][D->val[i]];
			     j=D->firstVal[u2]; 
			     while((j<D->firstVal[u2]+D->nbVal_prof[prof][u2]) && D->compat[jc][D->posInComp[u2][D->val[j]]]!=1) j++;
			     if (j==D->firstVal[u2]+D->nbVal_prof[prof][u2])
			      { 
				//  there is no support for u1=D->val[i] in D[u2]
				last_pos(u1,D->val[i],prof,D,Gp);
				if(D->nbVal_prof[prof][u1]==0) return 0;
				turn=1;
			      }
			     else i++;
			    }
			  }
			}
		     }
		   }
		 }
	return 1;
}






int filter(int prof, Tdomain* D, Tgraph* Gp, Tgraph* Gt, int maxSize)
	// returns the number of variables that can be assigned w.r.t SoftAlldiff constraint at the depth prof in the search tree
{
	// check global SoftallDiff for all vertices of Gp
		
	int nbToRemove, nbMatched; 
	int toRemove[2*D->valSize];
	nbMatched=checkSoftAllDiff(Gp->nbVertices,Gt->nbVertices,D,prof,D->globalMatching,&nbToRemove,toRemove, maxSize);
  	return nbMatched;   
}


int filter_GAC(int prof, Tdomain* D, Tgraph* Gp, Tgraph* Gt, int maxSize)
	// returns the number of variables that can be assigned w.r.t SoftAlldiff constraint at the depth prof in the search tree
{
	int u, i;

	// check global softAllDiff for all vertices of Gp
		
	int nbToRemove, nbMatched; 
	int toRemove[2*D->valSize];
	nbMatched=checkSoftAllDiff_lazy(Gp->nbVertices,Gt->nbVertices,D,prof,D->globalMatching,&nbToRemove,toRemove, maxSize);
	         	    
	  if(nbMatched==maxSize+1)     
	    {
		for (i=0; i<nbToRemove; i++)
		  {
			u=toRemove[2*i];
			last_pos(u,toRemove[2*i+1],prof,D,Gp);
			nbFilteredSoftAllDiff++;
	    	  }
	    }
  	return nbMatched;   

}

